# Datei zur Erstellung der Bibliotheksdateien

# Alle Variablen fuer Hauptpfade
OBJECT_FILE_PATH = $(OUTPUT_FOLDER)/
LIBRARY_FILE_PATH = $(OUTPUT_FOLDER)/

CXX_FLAGS = -std=c++14 -fvisibility=hidden -Wno-deprecated-declarations -Wno-ignored-qualifiers

# Alle Variablen fuer Projektklassen
WORKSPACE_PATH = ../
PROJECT_PATH = ./

# Variable fuer gemeinsam verwendete Dateien.
SHARED_INCLUDE_PATH = $(WORKSPACE_PATH)shared/include/
SHARED_SRC_PATH = $(WORKSPACE_PATH)shared/src/
# Variable fuer 3rd libraries
THIRD_PATH = $(WORKSPACE_PATH)3rd/
# Variable fuer TinyXML
TINYXML2_PATH = $(THIRD_PATH)tinyxml2/9.0.0/install/
# Variable fuer Boost-Pfad.
BOOST_PATH = $(THIRD_PATH)boost/
# Variable fuer SimpleIni
SIMPLEINI_PATH = $(THIRD_PATH)simpleini/4.17/install/
# Variable fuer fmt-Library
FMT_PATH = $(THIRD_PATH)fmt/6.1.2/fmt-6.1.2/
# Variable fuer spdlog-Library
SPDLOG_PATH = $(THIRD_PATH)spdlog/1.5.0/spdlog-1.5.0/
# Variable fuer sqlite3
SQLITE_PATH = $(THIRD_PATH)sqlite/3.36.0/install/

# Benoetigte externe Bibliotheken
INPUT_LIBRARY_FILES = -lpthread -ldl

# Alle Variablen fuer Project
SOURCE_PATH = $(PROJECT_PATH)src/
SOURCE_FILES = \
$(PROJECT_PATH)src/datafox_library_iv.cpp \
$(PROJECT_PATH)src/dfcom_version.cpp 

# Fuer jedes Unterverzeichnis welches Quelltextdateien enthaelt wird ein separates target angelegt
$(OBJECT_FILE_PATH)%.o: $(SOURCE_PATH)%.cpp
	@echo 'Erstelle Datei: $<'
	$(GPP) $(CXX_FLAGS) -I$(WORKSPACE_PATH) -I$(PROJECT_PATH) -I$(PROJECT_PATH)include/ -I$(SHARED_INCLUDE_PATH) -I$(THIRD_PATH) -I$(TINYXML2_PATH) -I$(BOOST_PATH) -DFMT_HEADER_ONLY -I$(FMT_PATH)include/ -I$(SIMPLEINI_PATH)include/ -I$(SPDLOG_PATH)include/ -I$(SQLITE_PATH)include/ -DBOOST_BIND_GLOBAL_PLACEHOLDERS $(OPTIMIZE) $(DEBUGLEVEL) $(WARNLEVEL) -c -fmessage-length=0 -fPIC -MMD -MP -MF"$(@:%.o=%.d)" -MT"$(@:%.o=%.d)" -o"$@" "$<"
	@echo ' '


# Alle Variablen fuer SimpleIni
SOURCE_PATH = $(SIMPLEINI_PATH)src/
SOURCE_FILES += \
$(SOURCE_PATH)ConvertUTF.cpp

# Fuer jedes Unterverzeichnis welches Quelltextdateien enthaelt wird ein separates target angelegt
$(OBJECT_FILE_PATH)%.o: $(SOURCE_PATH)%.cpp
	@echo 'Erstelle Datei: $<'
	$(GPP) $(CXX_FLAGS) -I$(WORKSPACE_PATH) -I$(SHARED_INCLUDE_PATH) -I$(THIRD_PATH) -I$(SIMPLEINI_PATH)include/ $(OPTIMIZE) $(DEBUGLEVEL) $(WARNLEVEL) -Wimplicit-fallthrough=0 -c -fmessage-length=0 -fPIC -MMD -MP -MF"$(@:%.o=%.d)" -MT"$(@:%.o=%.d)" -o"$@" "$<"
	@echo ' '

# Alle Variablen fuer TINYXML2
SOURCE_PATH = $(TINYXML2_PATH)tinyxml2/
SOURCE_FILES += \
$(SOURCE_PATH)tinyxml2.cpp

# Fuer jedes Unterverzeichnis welches Quelltextdateien enthaelt wird ein separates target angelegt
$(OBJECT_FILE_PATH)%.o: $(SOURCE_PATH)%.cpp
	@echo 'Erstelle Datei: $<'
	$(GPP) $(CXX_FLAGS) -I$(WORKSPACE_PATH) -I$(SHARED_INCLUDE_PATH) -I$(THIRD_PATH) -I$(TINYXML2_PATH) $(OPTIMIZE) $(DEBUGLEVEL) $(WARNLEVEL) -c -fmessage-length=0 -fPIC -MMD -MP -MF"$(@:%.o=%.d)" -MT"$(@:%.o=%.d)" -o"$@" "$<"
	@echo ' '

# Alle Variablen fuer ZIP Utils
SOURCE_PATH = $(THIRD_PATH)zip_utils/

# Aus den Sourcen die benoetigten Dateien ermitteln.
SOURCE_FILES_CPP = $(notdir $(SOURCE_FILES))
SOURCE_FILES_O = $(addprefix $(OBJECT_FILE_PATH), $(SOURCE_FILES_CPP:.cpp=.o))
SOURCE_FILES_D = $(addprefix $(OBJECT_FILE_PATH), $(SOURCE_FILES_CPP:.cpp=.d))


# Alle Variablen fuer API
SOURCE_PATH = $(PROJECT_PATH)src/api/
SOURCE_FILES += \
$(SOURCE_PATH)df_access_control.cpp \
$(SOURCE_PATH)df_access_control_lists.cpp \
$(SOURCE_PATH)df_access_control_lists_v1.cpp \
$(SOURCE_PATH)df_active_mode.cpp \
$(SOURCE_PATH)df_api.cpp \
$(SOURCE_PATH)df_connection.cpp \
$(SOURCE_PATH)df_files.cpp \
$(SOURCE_PATH)df_fingerprint.cpp \
$(SOURCE_PATH)df_lists.cpp \
$(SOURCE_PATH)df_logging.cpp \
$(SOURCE_PATH)df_setup_lists.cpp \
$(SOURCE_PATH)df_setup_records.cpp \
$(SOURCE_PATH)df_system_records.cpp \
$(SOURCE_PATH)df_timeboy_lists.cpp 

# Fuer jedes Unterverzeichnis welches Quelltextdateien enthaelt wird ein separates target angelegt
$(OBJECT_FILE_PATH)%.o: $(SOURCE_PATH)%.cpp
	@echo 'Erstelle Datei: $<'
	$(GPP) $(CXX_FLAGS) -I$(WORKSPACE_PATH) -I$(PROJECT_PATH)include/ -I$(SHARED_INCLUDE_PATH) -I$(THIRD_PATH) -I$(TINYXML2_PATH) -I$(BOOST_PATH) -DFMT_HEADER_ONLY -I$(FMT_PATH)include/ -I$(SPDLOG_PATH)include/ -I$(SIMPLEINI_PATH)include/ -I$(SQLITE_PATH)include/ -DBOOST_BIND_GLOBAL_PLACEHOLDERS $(OPTIMIZE) $(DEBUGLEVEL) $(WARNLEVEL) -c -fmessage-length=0 -fPIC -MMD -MP -MF"$(@:%.o=%.d)" -MT"$(@:%.o=%.d)" -o"$@" "$<"
	@echo ' '

# Alle Variablen fuer Allgemeines
SOURCE_PATH = $(SHARED_SRC_PATH)
SOURCE_FILES += \
$(SOURCE_PATH)document_studio.cpp

# Fuer jedes Unterverzeichnis welches Quelltextdateien enthaelt wird ein separates target angelegt
$(OBJECT_FILE_PATH)%.o: $(SOURCE_PATH)%.cpp
	@echo 'Erstelle Datei: $<'
	$(GPP) $(CXX_FLAGS) -I$(WORKSPACE_PATH) -I$(PROJECT_PATH) -I$(SHARED_INCLUDE_PATH) -I$(THIRD_PATH) -I$(TINYXML2_PATH) -I$(BOOST_PATH) -I$(SIMPLEINI_PATH)include/ -I$(SQLITE_PATH)include/ -DBOOST_BIND_GLOBAL_PLACEHOLDERS $(OPTIMIZE) $(DEBUGLEVEL) $(WARNLEVEL) -c -fmessage-length=0 -fPIC -MMD -MP -MF"$(@:%.o=%.d)" -MT"$(@:%.o=%.d)" -o"$@" "$<"
	@echo ' '

# Alle Variablen fuer Kommunikationsklassen
SOURCE_PATH = $(SHARED_SRC_PATH)channel/
SOURCE_FILES += \
$(SOURCE_PATH)channel_1_2_buffer.cpp \
$(SOURCE_PATH)channel_1_2.cpp \
$(SOURCE_PATH)channel_4_buffer.cpp \
$(SOURCE_PATH)channel_4.cpp \
$(SOURCE_PATH)channel_bridge.cpp

# Fuer jedes Unterverzeichnis welches Quelltextdateien enthaelt wird ein separates target angelegt
$(OBJECT_FILE_PATH)%.o: $(SOURCE_PATH)%.cpp
	@echo 'Erstelle Datei: $<'
	$(GPP) $(CXX_FLAGS) -I$(WORKSPACE_PATH) -I$(SHARED_INCLUDE_PATH) -I$(THIRD_PATH) -I$(TINYXML2_PATH) -I$(BOOST_PATH) -DFMT_HEADER_ONLY -I$(FMT_PATH)include/ -I$(SPDLOG_PATH)include/ -I$(SIMPLEINI_PATH)include/ -I$(SQLITE_PATH)include/ -DBOOST_BIND_GLOBAL_PLACEHOLDERS $(OPTIMIZE) $(DEBUGLEVEL) $(WARNLEVEL) -c -fmessage-length=0 -fPIC -MMD -MP -MF"$(@:%.o=%.d)" -MT"$(@:%.o=%.d)" -o"$@" "$<"
	@echo ' '


# Alle Variablen fuer Allgemeines
SOURCE_PATH = $(SHARED_SRC_PATH)common/
SOURCE_FILES += \
$(SOURCE_PATH)crc.cpp \
$(SOURCE_PATH)device_info.cpp \
$(SOURCE_PATH)df4_crypt.cpp \
$(SOURCE_PATH)dfcom_utils.cpp \
$(SOURCE_PATH)firmware_options.cpp \
$(SOURCE_PATH)hardware_info.cpp \
$(SOURCE_PATH)md5.cpp \
$(SOURCE_PATH)software_info.cpp \
$(SOURCE_PATH)software_version.cpp \
$(SOURCE_PATH)tcpip_info.cpp \
$(SOURCE_PATH)device_update.cpp \
$(SOURCE_PATH)text_translate.cpp \
$(SOURCE_PATH)utils.cpp

# Fuer jedes Unterverzeichnis welches Quelltextdateien enthaelt wird ein separates target angelegt
$(OBJECT_FILE_PATH)%.o: $(SOURCE_PATH)%.cpp
	@echo 'Erstelle Datei: $<'
	$(GPP) $(CXX_FLAGS) -I$(WORKSPACE_PATH) -I$(SHARED_INCLUDE_PATH) -I$(THIRD_PATH) -I$(TINYXML2_PATH) -I$(BOOST_PATH) -I$(SIMPLEINI_PATH)include/ -I$(SQLITE_PATH)include/ -DBOOST_BIND_GLOBAL_PLACEHOLDERS $(OPTIMIZE) $(DEBUGLEVEL) $(WARNLEVEL) -c -fmessage-length=0 -fPIC -MMD -MP -MF"$(@:%.o=%.d)" -MT"$(@:%.o=%.d)" -o"$@" "$<"
	@echo ' '


# Alle Variablen fuer Kommunikationsklassen
SOURCE_PATH = $(SHARED_SRC_PATH)datafox/communication/
SOURCE_FILES += \
$(SOURCE_PATH)aes_crypt_session.cpp \
$(SOURCE_PATH)aes_crypt.cpp \
$(SOURCE_PATH)connection_base.cpp \
$(SOURCE_PATH)controller_segment.cpp \
$(SOURCE_PATH)module_application.cpp \
$(SOURCE_PATH)module_base.cpp \
$(SOURCE_PATH)module_hardware.cpp \
$(SOURCE_PATH)module_system.cpp \
$(SOURCE_PATH)module_table.cpp \
$(SOURCE_PATH)protocol.cpp \
$(SOURCE_PATH)protocol_base.cpp \
$(SOURCE_PATH)protocol_crypt.cpp \
$(SOURCE_PATH)protocol_docking_station_v1.cpp

# Fuer jedes Unterverzeichnis welches Quelltextdateien enthaelt wird ein separates target angelegt
$(OBJECT_FILE_PATH)%.o: $(SOURCE_PATH)%.cpp
	@echo 'Erstelle Datei: $<'
	$(GPP) $(CXX_FLAGS) -I$(WORKSPACE_PATH) -I$(SHARED_INCLUDE_PATH) -I$(THIRD_PATH) -I$(TINYXML2_PATH) -I$(BOOST_PATH) -DFMT_HEADER_ONLY -I$(FMT_PATH)include/ -I$(SPDLOG_PATH)include/ -DBOOST_BIND_GLOBAL_PLACEHOLDERS $(OPTIMIZE) $(DEBUGLEVEL) $(WARNLEVEL) -c -fmessage-length=0 -fPIC -MMD -MP -MF"$(@:%.o=%.d)" -MT"$(@:%.o=%.d)" -o"$@" "$<"
	@echo ' '


# Alle Variablen fuer Datafox File Dateien
SOURCE_PATH = $(SHARED_SRC_PATH)file/
SOURCE_FILES += \
$(SOURCE_PATH)dfcom_ini_data.cpp \
$(SOURCE_PATH)dfcom_ini_helper.cpp \
$(SOURCE_PATH)file_aes.cpp \
$(SOURCE_PATH)file_device.cpp \
$(SOURCE_PATH)file_device_text.cpp \
$(SOURCE_PATH)file_device_zip.cpp \
$(SOURCE_PATH)file_display_colors.cpp \
$(SOURCE_PATH)file_display_fonts.cpp \
$(SOURCE_PATH)file_display_grafics.cpp \
$(SOURCE_PATH)file_hardware_info.cpp \
$(SOURCE_PATH)file_ini.cpp \
$(SOURCE_PATH)file_intel_hex.cpp \
$(SOURCE_PATH)file_studio_setup_xml.cpp \
$(SOURCE_PATH)file_touch_keys.cpp \
$(SOURCE_PATH)file_wlan_config.cpp \
$(SOURCE_PATH)file_xml.cpp \
$(SOURCE_PATH)ini_file.cpp

# Fuer jedes Unterverzeichnis welches Quelltextdateien enthaelt wird ein separates target angelegt
$(OBJECT_FILE_PATH)%.o: $(SOURCE_PATH)%.cpp
	@echo 'Erstelle Datei: $<'
	$(GPP) $(CXX_FLAGS) -I$(WORKSPACE_PATH) -I$(SHARED_INCLUDE_PATH) -I$(THIRD_PATH) -I$(TINYXML2_PATH) -I$(BOOST_PATH) -DFMT_HEADER_ONLY -I$(FMT_PATH)include/ -I$(SIMPLEINI_PATH)include/ -I$(SQLITE_PATH)include/ -DBOOST_BIND_GLOBAL_PLACEHOLDERS $(OPTIMIZE) $(DEBUGLEVEL) $(WARNLEVEL) -c -fmessage-length=0 -fPIC -MMD -MP -MF"$(@:%.o=%.d)" -MT"$(@:%.o=%.d)" -o"$@" "$<"
	@echo ' '


# Alle Variablen fuer File interface
SOURCE_PATH = $(SHARED_SRC_PATH)file/interface/
SOURCE_FILES += \
$(SOURCE_PATH)data_flash.cpp \
$(SOURCE_PATH)device_update_classes.cpp \
$(SOURCE_PATH)exchange_classes.cpp \
$(SOURCE_PATH)hardware_info_items.cpp \
$(SOURCE_PATH)mcu_memory.cpp \
$(SOURCE_PATH)studio_setup_classes.cpp

# Fuer jedes Unterverzeichnis welches Quelltextdateien enthaelt wird ein separates target angelegt
$(OBJECT_FILE_PATH)%.o: $(SOURCE_PATH)%.cpp
	@echo 'Erstelle Datei: $<'
	$(GPP) $(CXX_FLAGS) -I$(WORKSPACE_PATH) -I$(PROJECT_PATH) -I$(SHARED_INCLUDE_PATH) -I$(THIRD_PATH) -I$(TINYXML2_PATH) -I$(BOOST_PATH) -I$(SIMPLEINI_PATH)include/ -I$(SQLITE_PATH)include/ -DBOOST_BIND_GLOBAL_PLACEHOLDERS $(OPTIMIZE) $(DEBUGLEVEL) $(WARNLEVEL) -c -fmessage-length=0 -fPIC -MMD -MP -MF"$(@:%.o=%.d)" -MT"$(@:%.o=%.d)" -o"$@" "$<"
	@echo ' '


# Alle Variablen fuer Datafox Items
SOURCE_PATH = $(SHARED_SRC_PATH)file/interface/dfitem/
SOURCE_FILES += \
$(SOURCE_PATH)dfitem_access_control.cpp \
$(SOURCE_PATH)dfitem_access_control_2.cpp \
$(SOURCE_PATH)dfitem_base.cpp \
$(SOURCE_PATH)dfitem_device.cpp \
$(SOURCE_PATH)dfitem_handling.cpp \
$(SOURCE_PATH)dfitem_input_chain.cpp \
$(SOURCE_PATH)dfitem_input_chain_field.cpp \
$(SOURCE_PATH)dfitem_io_module.cpp \
$(SOURCE_PATH)dfitem_mde.cpp \
$(SOURCE_PATH)dfitem_menu.cpp \
$(SOURCE_PATH)dfitem_menu_choice.cpp \
$(SOURCE_PATH)dfitem_table.cpp \
$(SOURCE_PATH)dfitem_table_field.cpp \
$(SOURCE_PATH)dfitem_table_list.cpp \
$(SOURCE_PATH)dfitem_table_record.cpp 

# Fuer jedes Unterverzeichnis welches Quelltextdateien enthaelt wird ein separates target angelegt
$(OBJECT_FILE_PATH)%.o: $(SOURCE_PATH)%.cpp
	@echo 'Erstelle Datei: $<'
	$(GPP) $(CXX_FLAGS) -I$(WORKSPACE_PATH) -I$(SHARED_INCLUDE_PATH) -I$(THIRD_PATH) -I$(TINYXML2_PATH) -I$(BOOST_PATH) $(OPTIMIZE) $(DEBUGLEVEL) $(WARNLEVEL) -c -fmessage-length=0 -fPIC -MMD -MP -MF"$(@:%.o=%.d)" -MT"$(@:%.o=%.d)" -o"$@" "$<"
	@echo ' '


# Alle Variablen fuer Log
SOURCE_PATH = $(SHARED_SRC_PATH)log/
SOURCE_FILES += \
$(SOURCE_PATH)df_log.cpp \
$(SOURCE_PATH)df_log_check.cpp 

# Fuer jedes Unterverzeichnis welches Quelltextdateien enthaelt wird ein separates target angelegt
$(OBJECT_FILE_PATH)%.o: $(SOURCE_PATH)%.cpp
	@echo 'Erstelle Datei: $<'
	$(GPP) $(CXX_FLAGS) -I$(WORKSPACE_PATH) -I$(SHARED_INCLUDE_PATH) -I$(THIRD_PATH) -I$(TINYXML2_PATH) -I$(BOOST_PATH) $(OPTIMIZE) $(DEBUGLEVEL) $(WARNLEVEL) -c -fmessage-length=0 -fPIC -MMD -MP -MF"$(@:%.o=%.d)" -MT"$(@:%.o=%.d)" -o"$@" "$<"
	@echo ' '


# Alle Variablen fuer SQLITE3
SOURCE_PATH = $(SQLITE_PATH)
SOURCE_FILES_C = \
$(SOURCE_PATH)src/sqlite3.c

# Fuer jedes Unterverzeichnis welches Quelltextdateien enthaelt wird ein separates target angelegt
$(OBJECT_FILE_PATH)%.o: $(SOURCE_PATH)src/%.c
	@echo 'Erstelle Datei: $<'
	$(GCC) -I$(SQLITE_PATH)include/ $(OPTIMIZE) $(DEBUGLEVEL) $(WARNLEVEL) -Wno-cast-function-type -Wimplicit-fallthrough=0 -fvisibility=hidden -c -fmessage-length=0 -fPIC -MMD -MP -MF"$(@:%.o=%.d)" -MT"$(@:%.o=%.d)" -o"$@" "$<"
	@echo ' '

# Aus den Sourcen die benoetigten Dateien ermitteln.
SOURCE_FILES_Cx = $(notdir $(SOURCE_FILES_C))
SOURCE_FILES_O += $(addprefix $(OBJECT_FILE_PATH), $(SOURCE_FILES_Cx:.c=.o))
SOURCE_FILES_D += $(addprefix $(OBJECT_FILE_PATH), $(SOURCE_FILES_Cx:.c=.d))


# Static library
$(LIBRARY_FILE_PATH)DFCom.a: $(SOURCE_FILES_O)
	$(AR) -cr $(LIBRARY_FILE_PATH)DFCom.a $(SOURCE_FILES_O)
	$(RANLIB) $(LIBRARY_FILE_PATH)DFCom.a

# shared library
#
# Benoetigt zum ausfuehren ggf. Umgebungsvariable: LD_LIBRARY_PATH
# Pruefen ob vorhanden: echo $LD_LIBRARY_PATH
# setzen wenn fehlt:
#    LD_LIBRARY_PATH=/full/path/to/library/directory
#    export LD_LIBRARY_PATH
# oder ergaenzen wenn vorhanden:
#    LD_LIBRARY_PATH=/full/path/to/library/directory:${LD_LIBRARY_PATH}
#    export LD_LIBRARY_PATH
$(LIBRARY_FILE_PATH)libDFCom.so: $(SOURCE_FILES_O)
	$(GPP) -fvisibility=hidden -fPIC -shared $(SOURCE_FILES_O) $(INPUT_LIBRARY_FILES) -o $(LIBRARY_FILE_PATH)libDFCom.so

