/**
* @file  filedevicezip.h
* @brief Schnittstelle fuer die Klasse FileDeviceZip.
*/
//////////////////////////////////////////////////////////////////////
#ifndef FILEDEVICEZIP_H
#define FILEDEVICEZIP_H

#ifdef _MSC_VER
#pragma warning(disable:4786)
#endif

// Basisklasse einbinden
#include "file_device.h"

// Fuer die Verwendung von StructHardwareInfo
#include "interface/studio_setup_structs.h"

#if defined(_MSC_VER) || defined(__MINGW32__)
#include <direct.h>
#include <io.h>

#include <time.h>
#include <string>
#include <set>
#include <functional>

#include <algorithm>
#include <iterator>

typedef std::greater<std::string> sf;
typedef std::multiset<std::string, sf> ms;
typedef ms::iterator msIterator;

// Initialisierungsdatei des Zip-Archiv
#include "ini_file.h"
// MD5 Berechnung 
#include "common/md5.h"
// Routinen fuer die zip Unterstuetzung
#include "zip_utils/zip.h"
using namespace Zip;

#else

#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <unistd.h>
#include <string.h>

#include "SimpleIni.h"
#endif

// forward declaration
//////////////////////////////
class DeviceUpdate;


class FileDeviceZip : public FileDevice
{
public:
    FileDeviceZip();
    virtual ~FileDeviceZip();

    void resetData();

    int CreateZipFromDirectory( char *path, char *fileZIP, char *defaultLng, const std::string &updateDBPath = std::string() );

    int CheckZip(const std::string &filePath);
    int SearchInfosDir( std::string &filePath, const StructHardwareInfo &hwInfo, DeviceUpdate *deviceUpdate = nullptr );
    int SearchInfosZip( std::string &filePath, const StructHardwareInfo &hwInfo, DeviceUpdate *deviceUpdate = nullptr, std::function<void ( const char * )> trace = nullptr );
    int loadFirmwareFileZip( std::string &filePath, const std::string &fwFileName );
    int loadFirmwareFile( std::string &path, const std::string &fwFileName );

    int UnzipDefaultDFL(char *fileZIP, char *fileDefaultDFL);
    bool unzipUpdateDB( const std::string &fileZip, const std::string &expectedMD5, std::string &dbPath );

    void resetError();
    void setRecentZipError(const std::string &filePath, int lineNumber);

    void setErrorTextExtended( const std::string &text );
    void appendErrorTextExtended( const std::string &text );
    std::string errorTextExtended();

    bool updateDBAvailableZip( const std::string &filePath, std::string &expectedMD5, std::string &errorMsg );
    bool updateDBAvailable( const std::string &filePath, std::string &expectedMD5, std::string &errorMsg );

protected:
#if defined(_MSC_VER) || defined(__MINGW32__)
    CIniFile m_iniFile;
public:
    std::string m_version;
    std::string m_timestamp;
#else
    CSimpleIniA m_iniFile;
#endif

public:
    std::string m_errorExtended;
};

#endif
