#include "common/df4_crypt.h"

#if defined(_MSC_VER)
#pragma warning(disable : 4996)
#endif

#include <cstring>
#include <algorithm>

df4_crypt::df4_crypt()
{
}

df4_crypt::~df4_crypt()
{
}

void df4_crypt::load_crypt_key()
{
    std::array<unsigned char, 32> crypt_key = {
        0xc5, 0xc2, 0xc3, 0xc0, 0xc1, 0xd9, 0xdb, 0xf0, 
        0xcd, 0x07, 0xa7, 0xaf, 0xaa, 0x0a, 0x50, 0xae,
        0xc5, 0x4e, 0xc3, 0xd9, 0xaf, 0xd9, 0x09, 0xc1,
        0xa7, 0xef, 0xac, 0xc2, 0x0a, 0xbf, 0xcd, 0xae
    };

    m_binary_key = true;
    m_size = 32;
    m_key = crypt_key;
}

void df4_crypt::set_key( unsigned char* key, bool binary_key, unsigned char size )
{
    m_binary_key = binary_key;
    if ( binary_key )
    {
        if ( size > m_key.size() )
            size = 32;

        m_key.fill( 0 );
        m_size = size;
        std::copy_n( key, size, m_key.begin() );
    }
    else
    {
        m_key.fill( 0 );
        std::copy_if( key, key + m_key.size(), m_key.begin(), []( unsigned char v ) { return v != 0; } );
        m_size = 0;
    }
}

// ##################################################################
/**
* @brief Initialisiert die sbox fuer die Crypt-Verschlusselungsroutine.
*/
void df4_crypt::init()
{
    int i;
    int j;
    unsigned char* kptr = &m_key[0];
    unsigned char trans;

    for ( i = 0; i < 256; ++i )
    {
        m_sbox[i] = (unsigned char)i;
    }

    for ( i = 0; i < 256; ++i )
    {
        m_k[i] = *kptr;
        kptr++;

        if ( m_binary_key )
        {
            if( kptr == (&m_key[0] + m_size) )
            {
                kptr = &m_key[0];
            }
        }
        else
        {
            if ( *kptr == 0 )
            {
                kptr = &m_key[0];
            }
        }
    }

    for ( i = 0, j = 0; i < 256; ++i )
    {
        j = (j + m_sbox[i] + m_k[i]) % 256;
        trans = m_sbox[i];
        m_sbox[i] = m_sbox[j];
        m_sbox[j] = trans;
    }
}

// ##################################################################
/**
* @brief Ver-/Entschluesselt die uebergebenen Daten.
*/
void df4_crypt::crypt( unsigned char* data, std::size_t size )

{
    std::size_t i;
    std::size_t j;
    std::size_t t;
    std::size_t pos;
    unsigned char trans;

    // Kopie von sbox, da sie unten manipuliert wuerde.
    m_k = m_sbox;

    // Wenn die Angabe einer Laenge fehlt, wird davon ausgegangen,
    // dass mit einer Zeichenkette gearbeitet wird.
    if ( size == 0 )
    {
        size = std::strlen( (const char *)data );
    }

    i = j = pos = 0;
    for( ; (size - pos) > 0; ++pos )
    {

        i = (i + 1) % 256;
        j = (j + m_k[i]) % 256;
        trans = m_k[i];
        m_k[i] = m_k[j];
        m_k[j] = trans;
        t = (m_k[i] + m_k[j]) % 256;
        data[pos] ^= m_k[t];
    }
}



