/****************************************************************************
**
** Copyright (C) Datafox GmbH.
** All rights reserved.
** Contact: Datafox GmbH (develop@datafox.de)
**
** This file is part of the DFComDLL module of the Datafox SDK.
**
****************************************************************************/

#include "datafox/communication/module_hardware.h"
#include <stdio.h>
#include <cstring>

// ##################################################################
/**
* @brief Initialisieren der Membervariablen.
*
* @param connectionNumber Verbindungsnummer des Verbindungsobjektes für z.B. Trace.
*/
Datafox::Communication::ModuleHardware::ModuleHardware( int32_t connectionNumber, uint8_t deviceAddress )
    : ModuleBase( connectionNumber, deviceAddress ) 
{

}

// ##################################################################
/**
* @brief Destruktor.
*
*/
Datafox::Communication::ModuleHardware::~ModuleHardware() 
{
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*/
void Datafox::Communication::ModuleHardware::errorModule( uint16_t module, uint16_t error, std::string &text )
{
    trace( "%s(%i):INFO(3): %s( %u, %u, %p )\n", __FILE__, __LINE__, __func__, module, error, text.c_str() );

    // Ist IV_MODULE_HARDWARE ?
    if ( module == IV_MODULE_HARDWARE )
    {

    }

    // Ermittlung des Fehlertextes an Basisklasse abgeben.
    ModuleBase::errorModule( module, error, text );
}


// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*/
void Datafox::Communication::ModuleHardware::errorFunction( uint16_t module, uint16_t func, uint16_t error, std::string &text )
{
    trace( "%s(%i):INFO(3): %s( %u, %u, %u, %p )\n", __FILE__, __LINE__, __func__, module, func, error, text.c_str() );

    if ( module == IV_MODULE_HARDWARE )
    {
        // Sonderfehler für Passwortschutz abhandeln.
        if ( error == 28 )
        {
            // Die Modulfunktionen liefern den Fehlercode 28, wenn kein oder ein falsches Passwort verwendet wurde.
            // Es ist in diesem Fall zuerst das richtige Passwort für die Kommunikation erforderlich.
            text = boost::str( boost::format( "( %u ): Korrektes Passwort für die Kommunikation erforderlich.") % error );
            return;
        }
        else if ( error == 29 )
        {
            // Die Modulfunktionen liefern den Fehlercode 29, wenn 3 mal ein falsches Passwort verwendet wurde.
            // Die Befehlsausführung ist danach für 60 Sekunden blockiert.
            text = boost::str( boost::format( "( %u ): Gerät ist aufgrund von fehlerhaften Passworteingaben für 60 Sekunden gesperrt.") % error );
            return;
        }

        switch ( func )
        {
        case IV_FUNCTION_PING:
            // Setzen/Ermitteln des Geraetetyp
        case IV_FUNCTION_SET_TYPE_DEVICE:
        case IV_FUNCTION_GET_TYPE_DEVICE:

            // Setzen/Ermitteln des Boardtyp (Redesignversion z.B.: 0x13) (in Nibbles)
        case IV_FUNCTION_SET_TYPE_BOARD:
        case IV_FUNCTION_GET_TYPE_BOARD:

            // Setzen/Ermitteln der Boardkomponente: RS485 fuer Kommunikation mit PC
        case IV_FUNCTION_SET_COMPONENT_RS485_0:
        case IV_FUNCTION_GET_COMPONENT_RS485_0:

            // Setzen/Ermitteln der Boardkomponente: TCP/IP
        case IV_FUNCTION_SET_TYPE_TCPIP:
        case IV_FUNCTION_GET_TYPE_TCPIP:

            // Setzen/Ermitteln der Boardkomponente: Modem MC35i/MC55
        case IV_FUNCTION_SET_TYPE_MODEM_MC:
        case IV_FUNCTION_GET_TYPE_MODEM_MC:

            // Setzen/Ermitteln der Boardkomponente: RS485 fuer Zutrittskontrolle
        case IV_FUNCTION_SET_COMPONENT_RS485_1:
        case IV_FUNCTION_GET_COMPONENT_RS485_1:

            // Setzen/Ermitteln des Transpondertyps
        case IV_FUNCTION_SET_TYPE_TRANSPONDER:
        case IV_FUNCTION_GET_TYPE_TRANSPONDER:

            // Setzen/Ermitteln des Displaytyp
        case IV_FUNCTION_SET_TYPE_DISPLAY:
        case IV_FUNCTION_GET_TYPE_DISPLAY:

            // Setzen/Ermitteln des Tastaturtyps
        case IV_FUNCTION_SET_TYPE_KEYPAD:
        case IV_FUNCTION_GET_TYPE_KEYPAD:

            // Setzen/Ermitteln der Multimediakarte
        case IV_FUNCTION_SET_TYPE_MMC:
        case IV_FUNCTION_GET_TYPE_MMC:

            // Setzen/Ermitteln des GPS - Typs
        case IV_FUNCTION_SET_TYPE_GPS:
        case IV_FUNCTION_GET_TYPE_GPS:

            // Setzen/Ermitteln der USB - Komponente
        case IV_FUNCTION_SET_COMPONENT_USB:
        case IV_FUNCTION_GET_COMPONENT_USB:

            // Setzen/Ermitteln der Hardwareconfigurationsnummer
        case IV_FUNCTION_SET_HW_CONFIG:
        case IV_FUNCTION_GET_HW_CONFIG:

            // Setzen/Ermitteln des Funkmodultyps
        case IV_FUNCTION_SET_TYPE_RADIO:
        case IV_FUNCTION_GET_TYPE_RADIO:

            // Setzen/Ermitteln des Funkuhrmodultyps
        case IV_FUNCTION_SET_TYPE_DCF77:
        case IV_FUNCTION_GET_TYPE_DCF77:

            // Setzen/Ermitteln der analogen Eingaenge - Komponente
        case IV_FUNCTION_SET_COMPONENT_IN_ANALOG:
        case IV_FUNCTION_GET_COMPONENT_IN_ANALOG:

            // Setzen/Ermitteln der digitalen Eingaenge - Komponente
        case IV_FUNCTION_SET_COMPONENT_IN_DIGITAL:
        case IV_FUNCTION_GET_COMPONENT_IN_DIGITAL:

            // Setzen/Ermitteln der digitalen Ausgaenge - Komponente
        case IV_FUNCTION_SET_COMPONENT_OUT_DIGITAL:
        case IV_FUNCTION_GET_COMPONENT_OUT_DIGITAL:

            // Setzen/Ermitteln des Fingerprint + Typs
        case IV_FUNCTION_SET_TYPE_FINGERPRINT:
        case IV_FUNCTION_GET_TYPE_FINGERPRINT:

        case IV_FUNCTION_SET_TYPE_EXTERNAL_READER:
        case IV_FUNCTION_GET_TYPE_EXTERNAL_READER:

            // Optionen schreiben/lesen
        case IV_FUNCTION_SET_OPTION_IOMODULE:
        case IV_FUNCTION_GET_OPTION_IOMODULE:

        case IV_FUNCTION_SET_OPTION_COMSERVER:
        case IV_FUNCTION_GET_OPTION_COMSERVER:

        case IV_FUNCTION_SET_OPTION_TBADAPTION:
        case IV_FUNCTION_GET_OPTION_TBADAPTION:
            text = boost::str( boost::format( "( %u ): Nicht eingetragener Funktionsfehler!") % error );
            break;

        default:
            text = boost::str( boost::format( "( %u ): Nicht eingetragener Fehlerblock für Funktion!") % error );
            break;
        }

        return;
    }

    // Ermittlung des Fehlertextes an Basisklasse abgeben.
    ModuleBase::errorFunction( module, func, error, text );
}

//#########################################################
//###################### HARDWARE #########################
//#########################################################

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceAddress Anzusprechende Geräteadresse.
*/
int32_t Datafox::Communication::ModuleHardware::remoteCall( uint16_t module, uint16_t func, uint8_t *data, uint16_t &count )
{
    if ( module == IV_MODULE_HARDWARE )
    {
        switch ( func )
        {
        case IV_FUNCTION_PING:
            return ping( deviceNumberSender(), data, &count );

            // Setzen/Ermitteln des Geraetetyp
        case IV_FUNCTION_SET_TYPE_DEVICE:
            return writeDeviceType( deviceNumberSender(), data[0], *(uint16_t*)&data[1] );

        case IV_FUNCTION_GET_TYPE_DEVICE:
            return readDeviceType( deviceNumberSender(), (uint8_t&)data[0] );

            // Setzen/Ermitteln des Boardtyp (Redesignversion z.B.: 0x13) (in Nibbles)
        case IV_FUNCTION_SET_TYPE_BOARD:
            return writeBoardType( deviceNumberSender(), data[0], *(uint16_t*)&data[1] );

        case IV_FUNCTION_GET_TYPE_BOARD:
            return readBoardType( deviceNumberSender(), (uint8_t&)data[0] );

            // Setzen/Ermitteln der Boardkomponente: RS485 fuer Kommunikation mit PC
        case IV_FUNCTION_SET_COMPONENT_RS485_0:
            return writeRs485CommunicationComponent( deviceNumberSender(), data[0], *(uint16_t*)&data[1] );

        case IV_FUNCTION_GET_COMPONENT_RS485_0:
            return readRs485CommunicationComponent( deviceNumberSender(), (uint8_t&)data[0] );

            // Setzen/Ermitteln der Boardkomponente: TCP/IP
        case IV_FUNCTION_SET_TYPE_TCPIP:
            return writeNetworkAdapterType( deviceNumberSender(), data[0], *(uint16_t*)&data[1] );

        case IV_FUNCTION_GET_TYPE_TCPIP:
            return readNetworkAdapterType( deviceNumberSender(), (uint8_t&)data[0] );

            // Setzen/Ermitteln der Boardkomponente: Modem MC35i/MC55
        case IV_FUNCTION_SET_TYPE_MODEM_MC:
            return writeModemType( deviceNumberSender(), data[0], *(uint16_t*)&data[1] );

        case IV_FUNCTION_GET_TYPE_MODEM_MC:
            return readModemType( deviceNumberSender(), (uint8_t&)data[0] );

            // Setzen/Ermitteln der Boardkomponente: RS485 fuer Zutrittskontrolle
        case IV_FUNCTION_SET_COMPONENT_RS485_1:
            return writeRs485AccessControlComponent( deviceNumberSender(), data[0], *(uint16_t*)&data[1] );

        case IV_FUNCTION_GET_COMPONENT_RS485_1:
            return readRs485AccessControlComponent( deviceNumberSender(), (uint8_t&)data[0] );

            // Setzen/Ermitteln des Transpondertyps
        case IV_FUNCTION_SET_TYPE_TRANSPONDER:
            return writeTransponderReaderType( deviceNumberSender(), data[0], *(uint16_t*)&data[1] );

        case IV_FUNCTION_GET_TYPE_TRANSPONDER:
            return readTransponderReaderType( deviceNumberSender(), (uint8_t&)data[0] );

            // Setzen/Ermitteln des Displaytyp
        case IV_FUNCTION_SET_TYPE_DISPLAY:
            return writeDisplayType( deviceNumberSender(), data[0], *(uint16_t*)&data[1] );

        case IV_FUNCTION_GET_TYPE_DISPLAY:
            return readDisplayType( deviceNumberSender(), (uint8_t&)data[0] );

            // Setzen/Ermitteln des Tastaturtyps
        case IV_FUNCTION_SET_TYPE_KEYPAD:
            return writeKeypadType( deviceNumberSender(), data[0], *(uint16_t*)&data[1] );

        case IV_FUNCTION_GET_TYPE_KEYPAD:
            return readKeypadType( deviceNumberSender(), (uint8_t&)data[0] );

            // Setzen/Ermitteln der Multimediakarte
        case IV_FUNCTION_SET_TYPE_MMC:
            return writeMmcType( deviceNumberSender(), data[0], *(uint16_t*)&data[1] );

        case IV_FUNCTION_GET_TYPE_MMC:
            return readMmcType( deviceNumberSender(), (uint8_t&)data[0] );

            // Setzen/Ermitteln des GPS - Typs
        case IV_FUNCTION_SET_TYPE_GPS:
            return writeGpsType( deviceNumberSender(), data[0], *(uint16_t*)&data[1] );

        case IV_FUNCTION_GET_TYPE_GPS:
            return readGpsType( deviceNumberSender(), (uint8_t&)data[0] );

            // Setzen/Ermitteln der USB - Komponente
        case IV_FUNCTION_SET_COMPONENT_USB:
            return writeUsbComponent( deviceNumberSender(), data[0], *(uint16_t*)&data[1] );

        case IV_FUNCTION_GET_COMPONENT_USB:
            return readUsbComponent( deviceNumberSender(), (uint8_t&)data[0] );

            // Setzen/Ermitteln der Hardwareconfigurationsnummer
        case IV_FUNCTION_SET_HW_CONFIG:
            return writeHardwareInfoVersion( deviceNumberSender(), data[0], *(uint16_t*)&data[1] );

        case IV_FUNCTION_GET_HW_CONFIG:
            return readHardwareInfoVersion( deviceNumberSender(), (uint8_t&)data[0] );

            // Setzen/Ermitteln des Funkmodultyps
        case IV_FUNCTION_SET_TYPE_RADIO:
            return writeRadioType( deviceNumberSender(), data[0], *(uint16_t*)&data[1] );

        case IV_FUNCTION_GET_TYPE_RADIO:
            return readRadioType( deviceNumberSender(), (uint8_t&)data[0] );

            // Setzen/Ermitteln des Funkuhrmodultyps
        case IV_FUNCTION_SET_TYPE_DCF77:
            return writeDCF77Type( deviceNumberSender(), data[0], *(uint16_t*)&data[1] );

        case IV_FUNCTION_GET_TYPE_DCF77:
            return readDCF77Type( deviceNumberSender(), (uint8_t&)data[0] );

            // Setzen/Ermitteln der analogen Eingaenge - Komponente
        case IV_FUNCTION_SET_COMPONENT_IN_ANALOG:
            return writeAnalogInputComponent( deviceNumberSender(), *(uint32_t*)data, *(uint16_t*)&data[4] );

        case IV_FUNCTION_GET_COMPONENT_IN_ANALOG:
            return readAnalogInputComponent( deviceNumberSender(), (uint32_t&)data[0] );

            // Setzen/Ermitteln der digitalen Eingaenge - Komponente
        case IV_FUNCTION_SET_COMPONENT_IN_DIGITAL:
            return writeDigitalInputComponent( deviceNumberSender(), *(uint32_t*)data, *(uint16_t*)&data[4] );

        case IV_FUNCTION_GET_COMPONENT_IN_DIGITAL:
            return readDigitalInputComponent( deviceNumberSender(), (uint32_t&)data[0] );

            // Setzen/Ermitteln der digitalen Ausgaenge - Komponente
        case IV_FUNCTION_SET_COMPONENT_OUT_DIGITAL:
            return writeDigitalOutputComponent( deviceNumberSender(), *(uint32_t*)data, *(uint16_t*)&data[4] );

        case IV_FUNCTION_GET_COMPONENT_OUT_DIGITAL:
            return readDigitalOutputComponent( deviceNumberSender(), (uint32_t&)data[0] );

            // Setzen/Ermitteln des Fingerprint + Typs
        case IV_FUNCTION_SET_TYPE_FINGERPRINT:
            return writeFingerprintScannerType( deviceNumberSender(), data[0], *(uint16_t*)&data[1] );

        case IV_FUNCTION_GET_TYPE_FINGERPRINT:
            return readFingerprintScannerType( deviceNumberSender(), (uint8_t&)data[0] );

        case IV_FUNCTION_SET_TYPE_EXTERNAL_READER:
            return writeExternalTransponderReaderType( deviceNumberSender(), data[0], *(uint16_t*)&data[1] );

        case IV_FUNCTION_GET_TYPE_EXTERNAL_READER:
            return readExternalTransponderReaderType( deviceNumberSender(), (uint8_t&)data[0] );

            // Optionen schreiben/lesen
        case IV_FUNCTION_SET_OPTION_IOMODULE:
            return writeIoModuleOption( deviceNumberSender(), data[0], *(uint16_t*)&data[1] );

        case IV_FUNCTION_GET_OPTION_IOMODULE:
            return readIoModuleOption( deviceNumberSender(), (uint8_t&)data[0] );

        case IV_FUNCTION_SET_OPTION_COMSERVER:
            return writeComServerOption( deviceNumberSender(), data[0], *(uint16_t*)&data[1] );

        case IV_FUNCTION_GET_OPTION_COMSERVER:
            return readComServerOption( deviceNumberSender(), (uint8_t&)data[0] );

        case IV_FUNCTION_SET_OPTION_TBADAPTION:
            return writeTimeboyAdaptionOption( deviceNumberSender(), data[0], *(uint16_t*)&data[1] );

        case IV_FUNCTION_GET_OPTION_TBADAPTION:
            return readTimeboyAdaptionOption( deviceNumberSender(), (uint8_t&)data[0] );

        case IV_FUNCTION_INSPECTION_STATION:
        {
            uint8_t address;
            std::string answer( IV_PACKET_LENGTH_DATA, 0 );
            int32_t size_answer;
            int32_t to_pending = 0;
            int32_t res = inspectionStationCommand( deviceNumberSender(),
                                             data[0], (char *)&data[1],
                                            static_cast<uint16_t>( std::strlen( (const char *)&data[1] ) ),
                                             address, &answer[0], size_answer, to_pending );
            if ( res == IV_ERROR_SUCCESS )
            {
                data[0] = address;
                answer.resize( size_answer );
                answer.copy( (char *)&data[1], answer.size() );
                *(int32_t *)&data[1 + answer.size()] = size_answer;
            }

            return res;
        }

        case IV_FUNCTION_INSPECTION_STATION_OK:
            return replyInspectionStationCommand( deviceNumberSender() );

        case IV_FUNCTION_SET_OPTION_TPWRITE:
            return writeTransponderWriteOption( deviceNumberSender(), data[0], *(uint16_t *)&data[1] );

        case IV_FUNCTION_GET_OPTION_TPWRITE:
            return readTransponderWriteOption( deviceNumberSender(), (uint8_t &)data[0] );

        case IV_FUNCTION_SET_OPTION_HTTP:
            return writeHttpOption( deviceNumberSender(), data[0], *(uint16_t *)&data[1] );

        case IV_FUNCTION_GET_OPTION_HTTP:
            return readHttpOption( deviceNumberSender(), (uint8_t &)data[0] );

        case IV_FUNCTION_SET_OPTION_LANGUAGE:
            return writeLocalizationOption( deviceNumberSender(), data[0], *(uint16_t *)&data[1] );

        case IV_FUNCTION_GET_OPTION_LANGUAGE:
            return readLocalizationOption( deviceNumberSender(), (uint8_t &)data[0] );

        case IV_FUNCTION_SET_OPTION_BIOKEY_UPDATE:
            return writeFingerprintScannerUpdateOption( deviceNumberSender(), data[0], *(uint16_t *)&data[1] );

        case IV_FUNCTION_GET_OPTION_BIOKEY_UPDATE:
            return readFingerprintScannerUpdateOption( deviceNumberSender(), (uint8_t &)data[0] );

        case IV_FUNCTION_SET_SETUP_OPTIONS:
            return writeSetupOptions( deviceNumberSender(), *(uint16_t *)&data[0], *(uint16_t *)&data[2] );

        case IV_FUNCTION_GET_SETUP_OPTIONS:
            return readSetupOptions( deviceNumberSender(), (uint16_t &)data[0], (uint16_t &)data[2] );

        case IV_FUNCTION_SET_TYPE_TOUCH:
            return writeTouchscreenType( deviceNumberSender(), data[0], *(uint16_t *)&data[1] );

        case IV_FUNCTION_GET_TYPE_TOUCH:
            return readTouchscreenType( deviceNumberSender(), (uint8_t &)data[0] );

        case IV_FUNCTION_SET_TYPE_SENSOR:
            return writeSensorType( deviceNumberSender(), data[0], *(uint16_t *)&data[1] );

        case IV_FUNCTION_GET_TYPE_SENSOR:
            return readSensorType( deviceNumberSender(), (uint8_t &)data[0] );

        case IV_FUNCTION_SET_COMPONENT_BATTERY:
            return writeBatteryComponent( deviceNumberSender(), data[0], *(uint16_t *)&data[1] );

        case IV_FUNCTION_GET_COMPONENT_BATTERY:
            return readBatteryComponent( deviceNumberSender(), (uint8_t &)data[0] );

        case IV_FUNCTION_SET_COMPONENT_CHARGE:
            return writeBatteryChargerComponent( deviceNumberSender(), data[0], *(uint16_t *)&data[1] );

        case IV_FUNCTION_GET_COMPONENT_CHARGE:
            return readBatteryChargerComponent( deviceNumberSender(), (uint8_t &)data[0] );

        case IV_FUNCTION_GET_HARDWARE_INFO:
            return readHardwareInfo( deviceNumberSender(), (StructHardwareInfo &)data[0] );

        case IV_FUNCTION_SET_PAYABLE_OPTIONS:
            return writeBuyOptions( deviceNumberSender(), data[0], data[1], *(uint16_t *)&data[2] );

        case IV_FUNCTION_GET_PAYABLE_OPTIONS:
            return readBuyOptions( deviceNumberSender(), data[0], data[2] );

        default: // Funktion nicht definiert
            return 0x0200;
        }
    }

    return ModuleBase::remoteCall( module, func, data, count );
}

// ##################################################################
/**
* @brief Ermitteln des Geraetetyps.
*
* @param deviceAddress Anzusprechende Geräteadresse.
* @param val Geraetetyp
*             0 = PZE - Master IV
*             1 = Zutritts - Master IV
*             2 = BDE - Master IV
*           ... = ...
*/
int32_t Datafox::Communication::ModuleHardware::readDeviceType( uint8_t deviceAddress, uint8_t &val )
{
    trace( "%s(%i):INFO(3): %s( %u, %u )\n", __FILE__, __LINE__, __func__, deviceAddress, val );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_DEVICE, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceAddress, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_DEVICE, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof( uint8_t ) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__ );
            val = 0;
            return result;
        }

        // Wert uebernehmen
        val = (m_packetData[0] != 0xFF) ? m_packetData[0] : 0;
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int32_t result;
        // Parameterwert und -groesse setzen
        uint8_t type = get_property( "hardware_info.type_device", IV_VALUE_TYPE_DEVICE_UNKNOWN );
        count = sizeof(uint8_t);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_DEVICE, &type, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt die vorliegende Board-Revisionsnummer.
*
* @param deviceAddress Anzusprechende Geräteadresse.
* @param val Revisionsnummer in der Form (HiNibble Majornummer).(LoNibble Minornummer)
*             z. B. 0x14 -> Major 1, Minor (Redesign Nr.) 4
*/
int32_t Datafox::Communication::ModuleHardware::readBoardType(uint8_t deviceAddress, uint8_t &val)
{
    trace( "%s(%i):INFO(3): %s( %u, %u )\n", __FILE__, __LINE__, __func__, deviceAddress, val );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_BOARD, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceAddress, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_BOARD, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(uint8_t) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return result;
        }

        // Wert uebernehmen
        val = (m_packetData[0] != 0xFF) ? m_packetData[0] : 0;
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int32_t result;
        // Parameterwert und -groesse setzen
        uint8_t type = get_property( "hardware_info.type_board", 0 );
        count = sizeof(uint8_t);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_BOARD, &type, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt ob die bezeichnete Komponente enthalten ist.
*
* @param deviceAddress Anzusprechende Geräteadresse.
* @param val Wert ob Komponente enthalten = 1 / nicht enthalten = 0
*/
int32_t Datafox::Communication::ModuleHardware::readRs485CommunicationComponent(uint8_t deviceAddress, uint8_t &val)
{
    trace( "%s(%i):INFO(3): %s( %u, %u )\n", __FILE__, __LINE__, __func__, deviceAddress, val );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_RS485_0, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceAddress, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_RS485_0, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(uint8_t) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return result;
        }

        // Wert uebernehmen
        val = (m_packetData[0] != 0xFF) ? m_packetData[0] : 0;
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int32_t result;
        // Parameterwert und -groesse setzen
        uint8_t val = get_property( "hardware_info.component_rs485_0", 0 );
        count = sizeof(uint8_t);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_RS485_0, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt ob die bezeichnete Komponente enthalten ist.
*
* @param deviceAddress Anzusprechende Geräteadresse.
* @param val Ermittelte Variante.
*              0 = Keines
*              1 = MC35i
*              2 = MC55
*            ... = ...
*/
int32_t Datafox::Communication::ModuleHardware::readNetworkAdapterType( uint8_t deviceAddress, uint8_t &val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceAddress, val );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_TCPIP, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceAddress, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_TCPIP, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(uint8_t) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return result;
        }

        // Wert uebernehmen
        val = (m_packetData[0] != 0xFF) ? m_packetData[0] : 0;
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int32_t result;
        // Parameterwert und -groesse setzen
        uint8_t val = get_property( "hardware_info.type_tcpip", 0 );
        count = sizeof(uint8_t);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_TCPIP, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceAddress Anzusprechende Geräteadresse.
* @param val Ermittelte Variante.
*              0 = Keines
*              1 = MC35i
*              2 = MC55
*            ... = ...
*/
int32_t Datafox::Communication::ModuleHardware::readModemType( uint8_t deviceAddress, uint8_t &val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceAddress, val );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_MODEM_MC, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceAddress, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_MODEM_MC, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(uint8_t) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return result;
        }

        // Wert uebernehmen
        val = (m_packetData[0] != 0xFF) ? m_packetData[0] : 0;
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int32_t result;
        // Parameterwert und -groesse setzen
        uint8_t val = get_property( "hardware_info.type_modem_mc", 0 );
        count = sizeof(uint8_t);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_MODEM_MC, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt ob die bezeichnete Komponente enthalten ist.
*
* @param deviceAddress Anzusprechende Geräteadresse.
* @param val Wert ob Komponente enthalten = 1 / nicht enthalten = 0
*/
int32_t Datafox::Communication::ModuleHardware::readRs485AccessControlComponent( uint8_t deviceAddress, uint8_t &val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceAddress, val );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_RS485_1, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceAddress, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_RS485_1, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(uint8_t) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return result;
        }

        // Wert uebernehmen
        val = (m_packetData[0] != 0xFF) ? m_packetData[0] : 0;
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int32_t result;
        // Parameterwert und -groesse setzen
        uint8_t val = get_property( "hardware_info.component_rs485_1", 0 );
        count = sizeof(uint8_t);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_RS485_1, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt welcher Typ der bezeichnete Komponente enthalten ist.
*
* @param deviceAddress Anzusprechende Geräteadresse.
* @param val Ermittelte Variante.
*             0 = Keines
*             1 = Lehner kleines
*             2 = Lehner grosses
*           ... = ...
*/
int32_t Datafox::Communication::ModuleHardware::readDisplayType( uint8_t deviceAddress, uint8_t &val, bool checked )
{
    (void)checked;
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceAddress, val );

    // Impementierungswechsel: Bootloaderversionen 00.04.00.00 und 00.04.00.01 verwalten die Werte anderst:
    // - Funktionen TypeTransponder liefern <-> Displaywert
    // - Funktionen TypeDisplay liefern <-> Keypadwert
    // - Funktionen TypeKeypad liefern <-> Transponderwert
    //
    // Die Verwechslung wird in der Ableitung dieser Moduleklasse abgefangen.

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_DISPLAY, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceAddress, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_DISPLAY, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(uint8_t) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return result;
        }

        // Wert uebernehmen
        val = (m_packetData[0] != 0xFF) ? m_packetData[0] : 0;
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int32_t result;
        // Parameterwert und -groesse setzen
        uint8_t val = get_property( "hardware_info.type_display", 0 );
        count = sizeof(uint8_t);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_DISPLAY, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt welcher Typ der bezeichnete Komponente enthalten ist.
*
* @param deviceAddress Anzusprechende Geräteadresse.
* @param val Ermittelte Variante.
*             0 = Keine
*             1 = PZE - Master IV
*             2 = BDE - Master IV
*           ... = ...
*/
int32_t Datafox::Communication::ModuleHardware::readKeypadType( uint8_t deviceAddress, uint8_t &val, bool checked )
{
    (void)checked;
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceAddress, val );

    // Impementierungswechsel: Bootloaderversionen 00.04.00.00 und 00.04.00.01 verwalten die Werte anderst:
    // - Funktionen TypeTransponder liefern <-> Displaywert
    // - Funktionen TypeDisplay liefern <-> Keypadwert
    // - Funktionen TypeKeypad liefern <-> Transponderwert
    //
    // Die Verwechslung wird in der Ableitung dieser Moduleklasse abgefangen.

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_KEYPAD, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceAddress, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_KEYPAD, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(uint8_t) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return result;
        }

        // Wert uebernehmen
        val = (m_packetData[0] != 0xFF) ? m_packetData[0] : 0;
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int32_t result;
        // Parameterwert und -groesse setzen
        uint8_t val = get_property( "hardware_info.type_keypad", 0 );
        count = sizeof(uint8_t);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_KEYPAD, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt welcher Typ der bezeichnete Komponente enthalten ist.
*
* @param deviceAddress Anzusprechende Geräteadresse.
* @param val Ermittelte Variante.
*             0 = Keiner
*             1 = TSR30
*             2 = TSR32
*             3 = PHG PRIMO 100
*             4 = Cavitec Mifare
*             5 = Dallas iButton
*           ... = ...
*/
int32_t Datafox::Communication::ModuleHardware::readTransponderReaderType( uint8_t deviceAddress, uint8_t &val, bool checked )
{
    (void)checked;
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceAddress, val );

    // Impementierungswechsel: Bootloaderversionen 00.04.00.00 und 00.04.00.01 verwalten die Werte anderst:
    // - Funktionen TypeTransponder liefern <-> Displaywert
    // - Funktionen TypeDisplay liefern <-> Keypadwert
    // - Funktionen TypeKeypad liefern <-> Transponderwert
    //
    // Die Verwechslung wird in der Ableitung dieser Moduleklasse abgefangen.

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_TRANSPONDER, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceAddress, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_TRANSPONDER, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(uint8_t) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return result;
        }

        // Wert uebernehmen
        val = (m_packetData[0] != 0xFF) ? m_packetData[0] : 0;
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int32_t result;
        // Parameterwert und -groesse setzen
        uint8_t val = get_property( "hardware_info.type_transponder", 0 );
        count = sizeof(uint8_t);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_TRANSPONDER, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt welcher Typ der bezeichnete Komponente enthalten ist.
*
* @param deviceAddress Anzusprechende Geräteadresse.
* @param val Ermittelte Variante.
*             0 = Keine
*             1 = Fest integriert
*             2 = Herausnehmbar
*           ... = ...
*/
int32_t Datafox::Communication::ModuleHardware::readMmcType( uint8_t deviceAddress, uint8_t &val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceAddress, val );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_MMC, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceAddress, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_MMC, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(uint8_t) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return result;
        }

        // Wert uebernehmen
        val = (m_packetData[0] != 0xFF) ? m_packetData[0] : 0;
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int32_t result;
        // Parameterwert und -groesse setzen
        uint8_t val = get_property( "hardware_info.type_mmc", 0 );
        count = sizeof(uint8_t);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_MMC, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt welcher Typ der bezeichnete Komponente enthalten ist.
*
* @param deviceAddress Anzusprechende Geräteadresse.
* @param val Ermittelte Variante.
*             0 = Keine
*             1 = Bluegiga (Bluetooth)
*           ... = ...
*/
int32_t Datafox::Communication::ModuleHardware::readRadioType( uint8_t deviceAddress, uint8_t &val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceAddress, val );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_RADIO, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceAddress, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_RADIO, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(uint8_t) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return result;
        }

        // Wert uebernehmen
        val = (m_packetData[0] != 0xFF) ? m_packetData[0] : 0;
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int32_t result;
        // Parameterwert und -groesse setzen
        uint8_t val = get_property( "hardware_info.type_radio", 0 );
        count = sizeof(uint8_t);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_RADIO, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt welcher Typ der bezeichneten Komponente enthalten ist.
*
* @param deviceAddress Anzusprechende Geräteadresse.
* @param val Ermittelte Variante.
*              0 = Kein
*              1 = Furuno GN-80
*            ... = ...
*/
int32_t Datafox::Communication::ModuleHardware::readGpsType( uint8_t deviceAddress, uint8_t &val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceAddress, val );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_GPS, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceAddress, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_GPS, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(uint8_t) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return result;
        }

        // Wert uebernehmen
        val = (m_packetData[0] != 0xFF) ? m_packetData[0] : 0;
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int32_t result;
        // Parameterwert und -groesse setzen
        uint8_t val = get_property( "hardware_info.type_gps", 0 );
        count = sizeof(uint8_t);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_GPS, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt ob die bezeichnete Komponente enthalten ist.
*
* @param deviceAddress Anzusprechende Geräteadresse.
* @param val Wert ob Komponente enthalten = 1 / nicht enthalten = 0
*/
int32_t Datafox::Communication::ModuleHardware::readUsbComponent( uint8_t deviceAddress, uint8_t &val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceAddress, val );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_USB, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceAddress, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_USB, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(uint8_t) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return result;
        }

        // Wert uebernehmen
        val = (m_packetData[0] != 0xFF) ? m_packetData[0] : 0;
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int32_t result;
        // Parameterwert und -groesse setzen
        uint8_t val = get_property( "hardware_info.component_usb", 0 );
        count = sizeof(uint8_t);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_USB, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt ob die bezeichnete Komponente enthalten ist.
*
* @param deviceAddress Anzusprechende Geräteadresse.
* @param val Wert ob Komponente enthalten = 1 / nicht enthalten = 0
*/
int32_t Datafox::Communication::ModuleHardware::readBatteryComponent( uint8_t deviceAddress, uint8_t &val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceAddress, val );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_BATTERY, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceAddress, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_BATTERY, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(uint8_t) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return result;
        }

        // Wert uebernehmen
        val = (m_packetData[0] != 0xFF) ? m_packetData[0] : 0;
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int32_t result;
        // Parameterwert und -groesse setzen
        uint8_t val = get_property( "hardware_info.component_battery", 0 );
        count = sizeof(uint8_t);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_BATTERY, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace("%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt ob die bezeichnete Komponente enthalten ist.
*
* @param deviceAddress Anzusprechende Geräteadresse.
* @param val Wert ob Komponente enthalten = 1 / nicht enthalten = 0
*/
int32_t Datafox::Communication::ModuleHardware::readBatteryChargerComponent( uint8_t deviceAddress, uint8_t &val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceAddress, val );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_CHARGE, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceAddress, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_CHARGE, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(uint8_t) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return result;
        }

        // Wert uebernehmen
        val = (m_packetData[0] != 0xFF) ? m_packetData[0] : 0;
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int32_t result;
        // Parameterwert und -groesse setzen
        uint8_t val = get_property( "hardware_info.component_charge", 0 );
        count = sizeof(uint8_t);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_CHARGE, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt welcher Typ der bezeichnete Komponente enthalten ist.
*
* @param deviceAddress Anzusprechende Geräteadresse.
* @param val Ermittelte Variante.
*             0 = Keine
*             1 = HKW
*             4 = Nicht zu verwenden, da auf 0 gemappt.
*           ... = ...
*/
int32_t Datafox::Communication::ModuleHardware::readDCF77Type( uint8_t deviceAddress, uint8_t &val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceAddress, val );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_DCF77, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceAddress, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_DCF77, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(uint8_t) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return result;
        }

        // Wert uebernehmen
        // Wert wurde von FWV 04.01.01 fuer die Baudrate von Bluetooth verwendet und
        // steht nach initialisierung auf dem Wert 4. Dieser Wert, wird als Typ ignoriert.
        val = (m_packetData[0] != 0xFF && m_packetData[0] != 4) ? m_packetData[0] : 0;
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int32_t result;
        // Parameterwert und -groesse setzen
        uint8_t val = get_property( "hardware_info.type_dfc77", 0 );
        count = sizeof(uint8_t);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_DCF77, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt ob die bezeichnete Komponente enthalten ist.
*
* @param deviceAddress Anzusprechende Geräteadresse.
* @param val Wert ob Komponente enthalten = 1 / nicht enthalten = 0
*/
int32_t Datafox::Communication::ModuleHardware::readAnalogInputComponent( uint8_t deviceAddress, uint32_t& val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceAddress, val );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_IN_ANALOG, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceAddress, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_IN_ANALOG, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(uint32_t) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return result;
        }

        // Wert uebernehmen
        val = (*(uint32_t*)m_packetData != 0xFFFFFFFF) ? *(uint32_t*)m_packetData : 0;
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int32_t result;
        // Parameterwert und -groesse setzen
        uint32_t val = get_property( "hardware_info.component_in_analog", 0 );
        count = sizeof(uint32_t);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_IN_ANALOG, (uint8_t*)&val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt ob die bezeichnete Komponente enthalten ist.
*
* @param deviceAddress Anzusprechende Geräteadresse.
* @param val Wert ob Komponente enthalten = 1 / nicht enthalten = 0
*/
int32_t Datafox::Communication::ModuleHardware::readDigitalInputComponent( uint8_t deviceAddress, uint32_t& val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceAddress, val );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_IN_DIGITAL, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceAddress, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_IN_DIGITAL, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(uint32_t) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return result;
        }

        // Wert uebernehmen
        val = (*(uint32_t*)m_packetData != 0xFFFFFFFF) ? *(uint32_t*)m_packetData : 0;
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int32_t result;
        // Parameterwert und -groesse setzen
        uint32_t val = get_property( "hardware_info.component_in_digital", 0 );
        count = sizeof(uint32_t);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_IN_DIGITAL, (uint8_t*)&val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt ob die bezeichnete Komponente enthalten ist.
*
* @param deviceAddress Anzusprechende Geräteadresse.
* @param val Wert ob Komponente enthalten = 1 / nicht enthalten = 0
*/
int32_t Datafox::Communication::ModuleHardware::readDigitalOutputComponent( uint8_t deviceAddress, uint32_t& val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceAddress, val );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_OUT_DIGITAL, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceAddress, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_OUT_DIGITAL, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(uint32_t) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return result;
        }

        // Wert uebernehmen
        val = (*(uint32_t*)m_packetData != 0xFFFFFFFF) ? *(uint32_t*)m_packetData : 0;
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int32_t result;
        // Parameterwert und -groesse setzen
        uint32_t val = get_property( "hardware_info.component_out_digital", 0 );
        count = sizeof(uint32_t);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_OUT_DIGITAL, (uint8_t*)&val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt ob die bezeichnete Option enthalten ist.
*
* @param deviceAddress Anzusprechende Geräteadresse.
* @param val Wert ob Option enthalten = 1 / nicht enthalten = 0
*/
int32_t Datafox::Communication::ModuleHardware::readIoModuleOption( uint8_t deviceAddress, uint8_t &val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceAddress, val );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_IOMODULE, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceAddress, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_IOMODULE, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(uint8_t) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return result;
        }

        // Wert uebernehmen
        val = m_packetData[0];
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int32_t result;
        // Parameterwert und -groesse setzen
        uint8_t val = get_property( "hardware_info.option_io_module", 0 );
        count = sizeof(uint8_t);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_IOMODULE, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt ob die bezeichnete Option enthalten ist.
*
* @param deviceAddress Anzusprechende Geräteadresse.
* @param val Wert ob Option enthalten = 1 / nicht enthalten = 0
*/
int32_t Datafox::Communication::ModuleHardware::readComServerOption( uint8_t deviceAddress, uint8_t &val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceAddress, val );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_COMSERVER, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceAddress, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_COMSERVER, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(uint8_t) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return result;
        }

        // Wert uebernehmen
        val = m_packetData[0];
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int32_t result;
        // Parameterwert und -groesse setzen
        uint8_t val = get_property( "hardware_info.option_com_server", 0 );
        count = sizeof(uint8_t);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_COMSERVER, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace("%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt ob die bezeichnete Option enthalten ist.
*
* @param deviceAddress Anzusprechende Geräteadresse.
* @param val Wert ob Option enthalten = 1 / nicht enthalten = 0
*/
int32_t Datafox::Communication::ModuleHardware::readTimeboyAdaptionOption( uint8_t deviceAddress, uint8_t &val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceAddress, val );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_TBADAPTION, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceAddress, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_TBADAPTION, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(uint8_t) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return result;
        }

        // Wert uebernehmen
        val = m_packetData[0];
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int32_t result;
        // Parameterwert und -groesse setzen
        uint8_t val = get_property( "hardware_info.option_tb_adaption", 0 );
        count = sizeof(uint8_t);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_TBADAPTION, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceAddress Anzusprechende Geräteadresse.
*/
int32_t Datafox::Communication::ModuleHardware::inspectionStationCommand( uint8_t deviceAddress, uint8_t address, char* data, int32_t sizeData, uint8_t &addressAnswer, char* dataAnswer, int32_t& sizeAnswer, int32_t toPending )
{
    trace( "%s(%i):INFO(3): %s( %i, %i, %p, %i )\n", __FILE__, __LINE__, __func__, deviceAddress, address, data, sizeData );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;
    int32_t i;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );

        // Parameter eintragen
        std::fill_n( m_packetData, sizeof( m_packetData ), 0 );
        // Erster Parameter ist die Adresse
        m_packetData[0] = address;
        count = sizeof(uint8_t);
        // Folgender Parameter ist eine Optionale Zeichenkette
        for ( i = 0; i < sizeData; ++i )
            m_packetData[count + i] = data[i];

        count += sizeData;
        // Nullterminierung sicherstellen
        if ( m_packetData[count] != '\0' )
        {
            count++;
            m_packetData[count] = '\0';
        }
        else
        {
            count++;
        }

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_INSPECTION_STATION, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceAddress, IV_MODULE_HARDWARE, IV_FUNCTION_INSPECTION_STATION, result );
            return result;
        }

        // OK
        if ( count )
        {
            trace( "%s(%i):WARNING(3): %s() [ Antwort enthaelt %i Byte an unnuetze Daten. ]\n", __FILE__, __LINE__, __func__, count );
        }

        // Aktive Antwort abwarten
        if ( (result = call( IV_CONTEXT_SLAVE, IV_MODULE_HARDWARE, IV_FUNCTION_INSPECTION_STATION_OK, m_packetData, count, ((toPending*1000) / connection().readTimeout()) )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceNumberReceiver(), IV_MODULE_HARDWARE, IV_FUNCTION_INSPECTION_STATION, result );
            return result;
        }

        // Erstes Byte ist die Adresse
        addressAnswer = m_packetData[0];
        // Folgend ein optionaler String
        for ( i = 0; i < (count-1) && i < sizeAnswer; ++i )
            dataAnswer[i] = m_packetData[1 + i];

        // Laenge der Zeichenkette
        sizeAnswer = i;
    }
    else 
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_INSPECTION_STATION, tmp, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceAddress Anzusprechende Geräteadresse.
*/
int32_t Datafox::Communication::ModuleHardware::replyInspectionStationCommand( uint8_t deviceAddress )
{
    trace( "%s(%i):INFO(3): %s( %i )\n", __FILE__, __LINE__, __func__, deviceAddress );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
#ifdef WITH_VIRTUAL_DEVICE
        trace( "%s(%i):ERROR(1): %s()\n", __FILE__, __LINE__, __func__ );
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }
    else
    {
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_INSPECTION_STATION_OK, tmp, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt ob die bezeichnete Option enthalten ist.
*
* @param deviceAddress Anzusprechende Geräteadresse.
* @param val Wert ob Option enthalten = 1 / nicht enthalten = 0
*/
int32_t Datafox::Communication::ModuleHardware::readTransponderWriteOption( uint8_t deviceAddress, uint8_t &val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceAddress, val );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_TPWRITE, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceAddress, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_TPWRITE, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(uint8_t) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return result;
        }

        // Wert uebernehmen
        val = m_packetData[0];
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int32_t result;
        // Parameterwert und -groesse setzen
        uint8_t val = get_property( "hardware_info.option_tp_write", 0 );
        count = sizeof(uint8_t);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_TPWRITE, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt ob die bezeichnete Option enthalten ist.
*
* @param deviceAddress Anzusprechende Geräteadresse.
* @param val Wert ob Option enthalten = 1 / nicht enthalten = 0
*/
int32_t Datafox::Communication::ModuleHardware::readHttpOption( uint8_t deviceAddress, uint8_t &val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceAddress, val );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_HTTP, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceAddress, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_HTTP, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(uint8_t) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return result;
        }

        // Wert uebernehmen
        val = m_packetData[0];
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int32_t result;
        // Parameterwert und -groesse setzen
        uint8_t val = get_property( "hardware_info.option_http", 0 );
        count = sizeof(uint8_t);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_HTTP, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt ob die bezeichnete Option enthalten ist.
*
* @param deviceAddress Anzusprechende Geräteadresse.
* @param val Wert ob Option enthalten = 1 / nicht enthalten = 0
*/
int32_t Datafox::Communication::ModuleHardware::readLocalizationOption( uint8_t deviceAddress, uint8_t &val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceAddress, val );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_LANGUAGE, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceAddress, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_LANGUAGE, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(uint8_t) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return result;
        }

        // Wert uebernehmen
        val = m_packetData[0];
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int32_t result;
        // Parameterwert und -groesse setzen
        uint8_t val = get_property( "hardware_info.option_language", 0 );
        count = sizeof(uint8_t);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_LANGUAGE, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt ob die bezeichnete Option enthalten ist.
*
* @param deviceAddress Anzusprechende Geräteadresse.
* @param val Wert ob Option enthalten = 1 / nicht enthalten = 0
*/
int32_t Datafox::Communication::ModuleHardware::readFingerprintScannerUpdateOption( uint8_t deviceAddress, uint8_t &val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceAddress, val );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_BIOKEY_UPDATE, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceAddress, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_BIOKEY_UPDATE, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(uint8_t) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return result;
        }

        // Wert uebernehmen
        val = m_packetData[0];
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int32_t result;
        // Parameterwert und -groesse setzen
        uint8_t val = get_property( "hardware_info.option_biokey_update", 0 );
        count = sizeof(uint8_t);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_BIOKEY_UPDATE, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt die bezahlbaren Optionen enthalten ist.
*
* @param deviceAddress Anzusprechende Geräteadresse.
* @param mask Maske, welche Optionen unterstuetzt werden.
* @param options Wert ob Option enthalten = 1 / nicht enthalten = 0
*/
int32_t Datafox::Communication::ModuleHardware::readBuyOptions( uint8_t deviceAddress, uint8_t &mask, uint8_t &options )
{
    trace( "%s(%i):INFO(3): %s( %i, 0x%02X, 0x%02X )\n", __FILE__, __LINE__, __func__, deviceAddress, mask, options );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_PAYABLE_OPTIONS, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceAddress, IV_MODULE_HARDWARE, IV_FUNCTION_GET_PAYABLE_OPTIONS, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != (2 * sizeof(uint8_t)) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            mask = 0;
            options = 0;
            return result;
        }

        // Werte uebernehmen
        mask = m_packetData[0];
        options = m_packetData[1];
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int32_t result;
        // Parameterwert und -groesse setzen
        uint8_t buf[2];
        buf[0] = get_property( "hardware_info.options_payable_mask", 0 );
        count = sizeof(uint8_t);
        buf[1] = get_property( "hardware_info.options_payable", 0 );
        count += sizeof(uint8_t);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_PAYABLE_OPTIONS, buf, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt welcher Typ der bezeichnete Komponente enthalten ist.
*
* @param deviceAddress Anzusprechende Geräteadresse.
* @param val Ermittelte Variante.
*             0 = Keine
*             1 = Idencom (BioKey 3000)
*           ... = ...
*/
int32_t Datafox::Communication::ModuleHardware::readFingerprintScannerType( uint8_t deviceAddress, uint8_t &val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceAddress, val );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_FINGERPRINT, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceAddress, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_FINGERPRINT, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(uint8_t) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return result;
        }

        // Wert uebernehmen
        val = (m_packetData[0] != 0xFF) ? m_packetData[0] : 0;
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int32_t result;
        // Parameterwert und -groesse setzen
        uint8_t val = get_property( "hardware_info.type_fingerprint", 0 );
        count = sizeof(uint8_t);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_FINGERPRINT, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt welcher Typ der bezeichnete Komponente enthalten ist.
*
* @param deviceAddress Anzusprechende Geräteadresse.
* @param val Bitfeld der verwendbaren Varianten.
*            -1 = Undefiniert
*            Bit 1 = GIS
*            Bit 2 = Protect
*            Bit 3 = PHG
*           ... = ...
*/
int32_t Datafox::Communication::ModuleHardware::readExternalTransponderReaderType( uint8_t deviceAddress, uint8_t &val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceAddress, val );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_EXTERNAL_READER, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceAddress, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_EXTERNAL_READER, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(uint8_t) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return result;
        }

        // BUGFIX
        // Eine Firmware die die Serien TS und Protect unterstützt setzt als unterstützte Serien
        // TS und Protect obwohl es so sein sollte, dass die angesprochene Serie eingetragen wird,
        // so dass der Wert der Hardwareinfo eindeutig ist.

        // Wert uebernehmen
        val = m_packetData[0];

        // Werden beide Flags vorgefunden wird von TS alleine ausgegangen.
        if ( (val != 0xFF) && (val & IV_VALUE_TYPE_EXTERNAL_READER_GIS) && (val & IV_VALUE_TYPE_EXTERNAL_READER_PROTECT) )
        {
            val = IV_VALUE_TYPE_EXTERNAL_READER_GIS;
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int32_t result;
        // Parameterwert und -groesse setzen
        uint8_t val = get_property( "hardware_info.type_external_reader", 0 );
        count = sizeof(uint8_t);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_EXTERNAL_READER, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt welcher Typ der bezeichneten Komponente enthalten ist.
*
* @param deviceAddress Anzusprechende Geräteadresse.
* @param val Ermittelte Variante.
*              0 = Kein
*              1 = Exklusiv-Line IV
*              2 = Flex-MasterIV
*            ... = ...
*/
int32_t Datafox::Communication::ModuleHardware::readTouchscreenType( uint8_t deviceAddress, uint8_t &val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceAddress, val );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_TOUCH, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceAddress, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_TOUCH, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(uint8_t) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return result;
        }

        // Wert uebernehmen
        val = (m_packetData[0] != 0xFF) ? m_packetData[0] : 0;
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int32_t result;
        // Parameterwert und -groesse setzen
        uint8_t val = get_property( "hardware_info.type_touch", 0 );
        count = sizeof(uint8_t);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_TOUCH, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt welcher Typ der bezeichneten Komponente enthalten ist.
*
* @param deviceAddress Anzusprechende Geräteadresse.
* @param val Ermittelte Variante.
*              0 = Kein
*              1 = Vibration
*              2 = Beschleunigung
*            ... = ...
*/
int32_t Datafox::Communication::ModuleHardware::readSensorType( uint8_t deviceAddress, uint8_t &val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceAddress, val );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_SENSOR, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceAddress, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_SENSOR, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(uint8_t) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return result;
        }

        // Wert uebernehmen
        val = (m_packetData[0] != 0xFF) ? m_packetData[0] : 0;
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int32_t result;
        // Parameterwert und -groesse setzen
        uint8_t val = get_property( "hardware_info.type_sensor", 0 );
        count = sizeof(uint8_t);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_SENSOR, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Setzen der Optionen fuer Setup.
*
* @param deviceAddress Anzusprechende Geräteadresse.
* @param mask Gibt die gültigen Werte des Wertes val an.
* @param val Gibt an wie die Option gesetzt werden soll.
*         bit 0 = Signalverarbeitung
*           ... = ...
*/
int32_t Datafox::Communication::ModuleHardware::writeSetupOptions( uint8_t deviceAddress, uint16_t mask, uint16_t options )
{
    trace( "%s(%i):INFO(3): %s( %i, 0x%04X, 0x%04X )\n", __FILE__, __LINE__, __func__, deviceAddress, mask, options );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );

        // Die Komponendenangabe uebernehmen.
        *(uint16_t*)&m_packetData[0] = mask;
        count += sizeof(uint16_t);

        *(uint16_t*)&m_packetData[2] = options;
        count += sizeof(uint16_t);

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_SETUP_OPTIONS, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceNumberReceiver(), IV_MODULE_HARDWARE, IV_FUNCTION_SET_SETUP_OPTIONS, result );
            return result;
        }

        // Keine Antwortdaten erwartet
        if ( count )
        {
            trace( "%s(%i):WARNING(3): %s() [ Antwort enthaelt %i Byte an unnuetze Daten. ]\n", __FILE__, __LINE__, __func__, count );
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int32_t result;
        uint8_t buf[4];
        // Parameterwert und -groesse setzen
        *reinterpret_cast<uint16_t*>( &buf[0] ) = get_property( "hardware_info.setup_options_mask", 0 );
        count = sizeof(uint16_t);
        *reinterpret_cast<uint16_t*>( &buf[count] ) = get_property( "hardware_info.setup_options", 0 );
        count += sizeof(uint16_t);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_SETUP_OPTIONS, buf, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermitteln der Optionen fuer Setup.
*
* @param deviceAddress Anzusprechende Geräteadresse.
* @param mask Gibt die gültigen Werte des Wertes val an.
* @param val Gibt an wie die Option gesetzt werden soll.
*         bit 0 = Signalverarbeitung
*           ... = ...
*/
int32_t Datafox::Communication::ModuleHardware::readSetupOptions( uint8_t deviceAddress, uint16_t& mask, uint16_t& options )
{
    trace( "%s(%i):INFO(3): %s( %i, 0x%04X, 0x%04X )\n", __FILE__, __LINE__, __func__, deviceAddress, mask, options );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_SETUP_OPTIONS, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceNumberReceiver(), IV_MODULE_HARDWARE, IV_FUNCTION_GET_SETUP_OPTIONS, result );
            return result;
        }

        // Keine Antwortdaten erwartet
        if ( count != (2 * sizeof(uint16_t)) )
        {
            trace( "%s(%i):WARNING(3): %s() [ Antwort enthaelt %i Byte an unnuetze Daten. ]\n", __FILE__, __LINE__, __func__, count );
        }

        // Die Komponendenangabe uebernehmen.
        mask = *(uint16_t*)&m_packetData[0];
        options = *(uint16_t*)&m_packetData[2];
    }
    else 
    {
#ifdef WITH_VIRTUAL_DEVICE
        int32_t result;
        uint8_t buf[4];
        // Parameterwert und -groesse setzen
        *reinterpret_cast<uint16_t*>( &buf[0] ) = get_property( "hardware_info.type_mmc", 0 );
        count = sizeof(uint8_t);
        *reinterpret_cast<uint16_t*>( &buf[count] ) = get_property( "hardware_info.type_mmc", 0 );
        count += sizeof(uint8_t);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_SETUP_OPTIONS, buf, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt die gesamte Hardwareinfo in einer einzelnen Struktur.
*
* @param deviceAddress Anzusprechende Geräteadresse.
* @param ebs Struktur mit Informationen zur Hardware.
*/
int32_t Datafox::Communication::ModuleHardware::readHardwareInfo( uint8_t deviceAddress, StructHardwareInfo& hwInfo )
{
    trace( "%s(%i):INFO(3): %s( %i )\n", __FILE__, __LINE__, __func__, deviceAddress );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_HARDWARE_INFO, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceAddress, IV_MODULE_HARDWARE, IV_FUNCTION_GET_HARDWARE_INFO, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count < (int32_t)sizeof(StructHardwareInfo) )
        {
            // Wenn nur die Bezahloptionen fehlen, dann trotzdem annehmen.
            // Uebergang von 04.02.02 auf 04.02.03 (29.05.2012)
            if (count != 0x2b)
                return 0x0300; // Es wurden weniger Daten gelesen als aktuell bekannt.

            m_packetData[count - 1] = 0; // Es fehlen noch die Bezahloptionen
            count++;
                
        }
        else if ( count > (int32_t)sizeof(StructHardwareInfo) )
        {
            // Es wurden mehr Daten gelesen als aktuell bekannt.
            return 0x0400;
            // Fehler 4
        }

        // Werte uebernehmen
        hwInfo = *((StructHardwareInfo*)&m_packetData[0]);
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int32_t result;
        // Parametergroesse setzen
        std::fill_n( reinterpret_cast<uint8_t*>( &hwInfo ), sizeof( StructHardwareInfo ), 0 );
        hwInfo.version               = get_property( "hardware_info.version", 0xFF );
        hwInfo.typeDevice            = get_property( "hardware_info.type_device", 0xFF );
        hwInfo.typeBoard             = get_property( "hardware_info.type_board", 0xFF );
        hwInfo.componentRS485_0      = get_property( "hardware_info.component_rs485_0", 0xFF );
        hwInfo.typeTCPIP             = get_property( "hardware_info.type_tcpip", 0xFF );
        hwInfo.typeModemMC           = get_property( "hardware_info.type_modem_mc", 0xFF );
        hwInfo.componentRS485_1      = get_property( "hardware_info.component_rs485_1", 0xFF );
        hwInfo.typeTransponder       = get_property( "hardware_info.type_transponder", 0xFF );
        hwInfo.typeDisplay           = get_property( "hardware_info.type_display", 0xFF );
        hwInfo.typeKeypad            = get_property( "hardware_info.type_keypad", 0xFF );
        hwInfo.typeMMC               = get_property( "hardware_info.type_mmc", 0xFF );
        hwInfo.typeGPS               = get_property( "hardware_info.type_gps", 0xFF );
        hwInfo.componentUSB          = get_property( "hardware_info.component_usb", 0xFF );
        hwInfo.typeRadio             = get_property( "hardware_info.type_radio", 0xFF );
        hwInfo.typeDCF77             = get_property( "hardware_info.type_dfc77", 0xFF );
        hwInfo.componentInAnalog     = get_property( "hardware_info.component_in_analog", 0xFFFFFFFF );
        hwInfo.componentInDigital    = get_property( "hardware_info.component_in_digital", 0xFFFFFFFF );
        hwInfo.componentOutDigital   = get_property( "hardware_info.component_out_digital", 0xFFFFFFFF );
        hwInfo.typeFingerprint       = get_property( "hardware_info.type_fingerprint", 0xFF );
        hwInfo.typeExternalReader    = get_property( "hardware_info.type_external_reader", 0xFF );
        hwInfo.optionIOModule        = get_property( "hardware_info.option_io_module", 0xFF );
        hwInfo.optionTBAdaption      = get_property( "hardware_info.option_tb_adaption", 0xFF );
        hwInfo.optionComserver       = get_property( "hardware_info.option_com_server", 0xFF );
        hwInfo.optionTPWrite         = get_property( "hardware_info.option_tp_write", 0xFF );
        hwInfo.optionHttp            = get_property( "hardware_info.option_http", 0xFF );
        hwInfo.optionLanguage        = get_property( "hardware_info.option_language", 0xFF );
        hwInfo.optionBiokeyUpdate    = get_property( "hardware_info.option_biokey_update", 0xFF );
        hwInfo.optionsSetup          = get_property( "hardware_info.option_setup", 0xFFFF );
        hwInfo.typeTouch             = get_property( "hardware_info.type_touch", 0xFF );
        hwInfo.typeSensor            = get_property( "hardware_info.type_sensor", 0xFF );
        hwInfo.componentBattery      = get_property( "hardware_info.component_battery", 0xFF );
        hwInfo.componentCharge       = get_property( "hardware_info.component_charge", 0xFF );
        hwInfo.optionsPayable        = get_property( "hardware_info.option_payable", 0xFF );
        count = sizeof(StructHardwareInfo);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_HARDWARE_INFO,
                             reinterpret_cast<uint8_t*>( &hwInfo ), count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt die "Version" der Hardwareconfiguration.
*
* @param deviceAddress Anzusprechende Geräteadresse.
* @param val Wert fuer Version der Hardwareconfiguration.
*/
int32_t Datafox::Communication::ModuleHardware::readHardwareInfoVersion( uint8_t deviceAddress, uint8_t &val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceAddress, val );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_HW_CONFIG, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceAddress, IV_MODULE_HARDWARE, IV_FUNCTION_GET_HW_CONFIG, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(uint8_t) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return result;
        }

        // Wert uebernehmen
        val = (m_packetData[0] != 0xFF) ? m_packetData[0] : 0;
    }
    else 
    {
#ifdef WITH_VIRTUAL_DEVICE
        int32_t result;
        // Parameterwert und -groesse setzen
        uint8_t val = get_property( "hardware_info.version", 0 );
        count = sizeof(uint8_t);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_HW_CONFIG, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}


#if !defined( DATAFOX_STUDIO_IV ) && !defined( DF_API_PRODUCTION ) && !defined( WITH_VIRTUAL_DEVICE )
// Funktions-Stumps

int32_t Datafox::Communication::ModuleHardware::randomPing( uint8_t deviceAddress )
{ 
    if ( !isOwnDeviceNumber( deviceAddress ) )
        return -IV_ERROR_FUNCTION_UNAVAILABLE;

    return 0x0200;
}

int32_t Datafox::Communication::ModuleHardware::writeDeviceType( uint8_t deviceAddress, uint8_t val, uint16_t val2 )
{ 
    (void)val; 
    (void)val2; 
    if ( !isOwnDeviceNumber( deviceAddress ) )
        return -IV_ERROR_FUNCTION_UNAVAILABLE;
    
    return 0x0200;
}

int32_t Datafox::Communication::ModuleHardware::writeBoardType( uint8_t deviceAddress, uint8_t val, uint16_t val2 )
{ 
    (void)val;
    (void)val2;
    if ( !isOwnDeviceNumber( deviceAddress ) )
        return -IV_ERROR_FUNCTION_UNAVAILABLE;
    
    return 0x0200;
}

int32_t Datafox::Communication::ModuleHardware::writeNetworkAdapterType( uint8_t deviceAddress, uint8_t val, uint16_t val2 )
{ 
    (void)val; 
    (void)val2; 
    if ( !isOwnDeviceNumber( deviceAddress ) )
        return -IV_ERROR_FUNCTION_UNAVAILABLE;
    
    return 0x0200;
}

int32_t Datafox::Communication::ModuleHardware::writeModemType( uint8_t deviceAddress, uint8_t val, uint16_t val2 )
{ 
    (void)val; 
    (void)val2; 
    if ( !isOwnDeviceNumber( deviceAddress ) )
        return -IV_ERROR_FUNCTION_UNAVAILABLE;
    
    return 0x0200;
}

int32_t Datafox::Communication::ModuleHardware::writeDisplayType( uint8_t deviceAddress, uint8_t val, uint16_t val2, bool checked )
{ 
    (void)val; 
    (void)val2; 
    (void)checked; 
    if ( !isOwnDeviceNumber( deviceAddress ) )
        return -IV_ERROR_FUNCTION_UNAVAILABLE; 
    
    return 0x0200;
}

int32_t Datafox::Communication::ModuleHardware::writeKeypadType( uint8_t deviceAddress, uint8_t val, uint16_t val2, bool checked )
{ 
    (void)val; 
    (void)val2; 
    (void)checked; 
    if ( !isOwnDeviceNumber( deviceAddress ) )
        return -IV_ERROR_FUNCTION_UNAVAILABLE; 
    
    return 0x0200;
}

int32_t Datafox::Communication::ModuleHardware::writeTransponderReaderType( uint8_t deviceAddress, uint8_t val, uint16_t val2, bool checked )
{ 
    (void)val; 
    (void)val2; 
    (void)checked; 
    if ( !isOwnDeviceNumber( deviceAddress ) )
        return -IV_ERROR_FUNCTION_UNAVAILABLE; 
    
    return 0x0200;
}

int32_t Datafox::Communication::ModuleHardware::writeMmcType( uint8_t deviceAddress, uint8_t val, uint16_t val2 )
{ 
    (void)val; 
    (void)val2; 
    if ( !isOwnDeviceNumber( deviceAddress ) )
        return -IV_ERROR_FUNCTION_UNAVAILABLE;
    
    return 0x0200;
}

int32_t Datafox::Communication::ModuleHardware::writeRadioType( uint8_t deviceAddress, uint8_t val, uint16_t val2 )
{ 
    (void)val; 
    (void)val2; 
    if ( !isOwnDeviceNumber( deviceAddress ) )
        return -IV_ERROR_FUNCTION_UNAVAILABLE;
    
    return 0x0200;
}

int32_t Datafox::Communication::ModuleHardware::writeGpsType( uint8_t deviceAddress, uint8_t val, uint16_t val2 )
{ 
    (void)val; 
    (void)val2; 
    if ( !isOwnDeviceNumber( deviceAddress ) )
        return -IV_ERROR_FUNCTION_UNAVAILABLE; 
    
    return 0x0200;
}

int32_t Datafox::Communication::ModuleHardware::writeDCF77Type( uint8_t deviceAddress, uint8_t val, uint16_t val2 )
{ 
    (void)val; 
    (void)val2; 
    if ( !isOwnDeviceNumber( deviceAddress ) )
        return -IV_ERROR_FUNCTION_UNAVAILABLE; 
    
    return 0x0200;
}

int32_t Datafox::Communication::ModuleHardware::writeFingerprintScannerType( uint8_t deviceAddress, uint8_t val, uint16_t val2 )
{ 
    (void)val; 
    (void)val2; 
    if ( !isOwnDeviceNumber( deviceAddress ) )
        return -IV_ERROR_FUNCTION_UNAVAILABLE; 
    
    return 0x0200;
}

int32_t Datafox::Communication::ModuleHardware::writeExternalTransponderReaderType( uint8_t deviceAddress, uint8_t val, uint16_t val2 ) 
{
    (void)val2;
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceAddress, val );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    uint16_t key;
    switch ( val ) 
    {
    case 0xFF:
        key = 0xCF38;
        break;

    case 0x00:
        key = 0x8F78;
        break;

    case 0x01:
        key = 0x4FB9;
        break;

    case 0x02:
        key = 0x4EF9;
        break;

    default:
        return -IV_ERROR_FUNCTION_PARAMETER;
    }

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );

        // Die Komponendenangabe uebernehmen.
        m_packetData[0] = val;
        count += sizeof(uint8_t);

        *(uint16_t*)&m_packetData[1] = key;
        count += sizeof(uint16_t);

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_TYPE_EXTERNAL_READER, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceNumberReceiver(), IV_MODULE_HARDWARE, IV_FUNCTION_SET_TYPE_EXTERNAL_READER, result );
            return result;
        }

        // Keine Antwortdaten erwartet
        if ( count )
        {
            trace( "%s(%i):WARNING(3): %s() [ Antwort enthaelt %i Byte an unnuetze Daten. ]\n", __FILE__, __LINE__, __func__, count );
        }
    }
    else 
    {
#ifdef WITH_VIRTUAL_DEVICE
        // Schluessel pruefen: Funktionsfehler
        if ( key != val2 )
            return 0x0600;

        int32_t result;
        // Parametergroesse setzen
        count = 0;

        // Wert an interne Struktur uebernehmen
        m_hardwareInfo.optionBiokeyUpdate = val;

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_TYPE_EXTERNAL_READER, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}
int32_t Datafox::Communication::ModuleHardware::writeTouchscreenType( uint8_t deviceAddress, uint8_t val, uint16_t val2 )
{ 
    (void)val;
    (void)val2;
    if ( !isOwnDeviceNumber( deviceAddress ) )
        return -IV_ERROR_FUNCTION_UNAVAILABLE;
    
    return 0x0200;
}

int32_t Datafox::Communication::ModuleHardware::writeSensorType( uint8_t deviceAddress, uint8_t val, uint16_t val2 )
{ 
    (void)val;
    (void)val2;
    if ( !isOwnDeviceNumber( deviceAddress ) )
        return -IV_ERROR_FUNCTION_UNAVAILABLE;
    
    return 0x0200;
}


int32_t Datafox::Communication::ModuleHardware::writeRs485CommunicationComponent( uint8_t deviceAddress, uint8_t val, uint16_t val2 )
{ 
    (void)val;
    (void)val2;
    if ( !isOwnDeviceNumber( deviceAddress ) )
        return -IV_ERROR_FUNCTION_UNAVAILABLE;
    
    return 0x0200;
}

int32_t Datafox::Communication::ModuleHardware::writeRs485AccessControlComponent( uint8_t deviceAddress, uint8_t val, uint16_t val2 )
{ 
    (void)val;
    (void)val2;
    if ( !isOwnDeviceNumber( deviceAddress ) )
        return -IV_ERROR_FUNCTION_UNAVAILABLE;
    
    return 0x0200;
}

int32_t Datafox::Communication::ModuleHardware::writeUsbComponent( uint8_t deviceAddress, uint8_t val, uint16_t val2 )
{ 
    (void)val;
    (void)val2;
    if ( !isOwnDeviceNumber( deviceAddress ) )
        return -IV_ERROR_FUNCTION_UNAVAILABLE;
    
    return 0x0200;
}

int32_t Datafox::Communication::ModuleHardware::writeAnalogInputComponent( uint8_t deviceAddress, uint32_t val, uint16_t val2 )
{ 
    (void)val; 
    (void)val2; 
    if ( !isOwnDeviceNumber( deviceAddress ) )
        return -IV_ERROR_FUNCTION_UNAVAILABLE;
    
    return 0x0200;
}

int32_t Datafox::Communication::ModuleHardware::writeDigitalInputComponent( uint8_t deviceAddress, uint32_t val, uint16_t val2 )
{ 
    (void)val; 
    (void)val2; 
    if ( !isOwnDeviceNumber( deviceAddress ) )
        return -IV_ERROR_FUNCTION_UNAVAILABLE;
    
    return 0x0200;
}

int32_t Datafox::Communication::ModuleHardware::writeDigitalOutputComponent( uint8_t deviceAddress, uint32_t val, uint16_t val2 )
{ 
    (void)val; 
    (void)val2; 
    if ( !isOwnDeviceNumber( deviceAddress ) )
        return -IV_ERROR_FUNCTION_UNAVAILABLE;
    
    return 0x0200;
}

int32_t Datafox::Communication::ModuleHardware::writeBatteryComponent( uint8_t deviceAddress, uint8_t val, uint16_t val2 )
{ 
    (void)val; 
    (void)val2; 
    if ( !isOwnDeviceNumber( deviceAddress ) )
        return -IV_ERROR_FUNCTION_UNAVAILABLE; 
    
    return 0x0200;
}

int32_t Datafox::Communication::ModuleHardware::writeBatteryChargerComponent( uint8_t deviceAddress, uint8_t val, uint16_t val2 )
{ 
    (void)val; 
    (void)val2; 
    if ( !isOwnDeviceNumber( deviceAddress ) )
        return -IV_ERROR_FUNCTION_UNAVAILABLE; 
    
    return 0x0200;
}

int32_t Datafox::Communication::ModuleHardware::writeIoModuleOption( uint8_t deviceAddress, uint8_t val, uint16_t val2 )     
{ 
    (void)val2;
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceAddress, val );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    uint16_t key;
    switch ( val )
    {
    case 0xFF:
        key = 0x0C59;
        break;

    case 0x00:
        key = 0x4C19;
        break;

    case 0x01:
        key = 0x8CD8;
        break;

    default:
        return -IV_ERROR_FUNCTION_PARAMETER;
    }

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );

        // Die Komponendenangabe uebernehmen.
        m_packetData[0] = val;
        count += sizeof(uint8_t);

        *(uint16_t*)&m_packetData[1] = key;
        count += sizeof(uint16_t);

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_IOMODULE, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceNumberReceiver(), IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_IOMODULE, result );
            return result;
        }

        // Keine Antwortdaten erwartet
        if ( count )
        {
            trace( "%s(%i):WARNING(3): %s() [ Antwort enthaelt %i Byte an unnuetze Daten. ]\n", __FILE__, __LINE__, __func__, count );
        }
    }
    else 
    {
#ifdef WITH_VIRTUAL_DEVICE
        // Schluessel pruefen: Funktionsfehler
        if ( key != val2 )
            return 0x0600;

        int32_t result;
        // Parametergroesse setzen
        count = 0;

        // Wert an interne Struktur uebernehmen
        m_hardwareInfo.optionIOModule = val;

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_IOMODULE, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

int32_t Datafox::Communication::ModuleHardware::writeComServerOption( uint8_t deviceAddress, uint8_t val, uint16_t val2 )    
{ 
    (void)val2;
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceAddress, val );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    uint16_t key;
    switch ( val )
    {
    case 0xFF:
        key = 0xCCF8;
        break;

    case 0x00:
        key = 0x8CB8;
        break;

    case 0x01:
        key = 0x4C79;
        break;

    default:
        return -IV_ERROR_FUNCTION_PARAMETER;
    }

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );

        // Die Komponendenangabe uebernehmen.
        m_packetData[0] = val;
        count += sizeof(uint8_t);

        *(uint16_t*)&m_packetData[1] = key;
        count += sizeof(uint16_t);

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_COMSERVER, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceNumberReceiver(), IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_COMSERVER, result );
            return result;
        }

        // Keine Antwortdaten erwartet
        if ( count )
        {
            trace( "%s(%i):WARNING(3): %s() [ Antwort enthaelt %i Byte an unnuetze Daten. ]\n", __FILE__, __LINE__, __func__, count );
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        // Schluessel pruefen: Funktionsfehler
        if ( key != val2 )
            return 0x0600;

        int32_t result;
        // Parametergroesse setzen
        count = 0;

        // Wert an interne Struktur uebernehmen
        m_hardwareInfo.optionComserver = val;

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_COMSERVER, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

int32_t Datafox::Communication::ModuleHardware::writeTimeboyAdaptionOption( uint8_t deviceAddress, uint8_t val, uint16_t val2 )   
{
    (void)val2;
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceAddress, val );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    uint16_t key;
    switch ( val )
    {
    case 0xFF:
        key = 0xCD18;
        break;

    case 0x00:
        key = 0x8D58;
        break;

    case 0x01:
        key = 0x4D99;
        break;

    default:
        return -IV_ERROR_FUNCTION_PARAMETER;
    }

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );

        // Die Komponendenangabe uebernehmen.
        m_packetData[0] = val;
        count += sizeof(uint8_t);

        *(uint16_t*)&m_packetData[1] = key;
        count += sizeof(uint16_t);

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_TBADAPTION, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceNumberReceiver(), IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_TBADAPTION, result );
            return result;
        }

        // Keine Antwortdaten erwartet
        if ( count )
        {
            trace( "%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten. ]\n", __FILE__, __LINE__, __func__, count );
        }
    }
    else 
    {
#ifdef WITH_VIRTUAL_DEVICE
        // Schluessel pruefen: Funktionsfehler
        if ( key != val2 )
            return 0x0600;

        int32_t result;
        // Parametergroesse setzen
        count = 0;

        // Wert an interne Struktur uebernehmen
        m_hardwareInfo.optionTBAdaption = val;

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_TBADAPTION, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

int32_t Datafox::Communication::ModuleHardware::writeTransponderWriteOption( uint8_t deviceAddress, uint8_t val, uint16_t val2 )   
{
    (void)val2;
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceAddress, val );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    uint16_t key;
    switch ( val )
    {
    case 0xFF:
        key = 0x0BD9;
        break;

    case 0x00:
        key = 0x4B99;
        break;

    case 0x01:
        key = 0x8B58;
        break;

    default:
        return -IV_ERROR_FUNCTION_PARAMETER;
    }
    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );

        // Die Komponendenangabe uebernehmen.
        m_packetData[0] = val;
        count += sizeof(uint8_t);

        *(uint16_t*)&m_packetData[1] = key;
        count += sizeof(uint16_t);

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_TPWRITE, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceNumberReceiver(), IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_TPWRITE, result );
            return result;
        }

        // Keine Antwortdaten erwartet
        if ( count )
        {
            trace( "%s(%i):WARNING(3): %s() [ Antwort enthaelt %i Byte an unnuetze Daten. ]\n", __FILE__, __LINE__, __func__, count );
        }
    }
    else 
    {
#ifdef WITH_VIRTUAL_DEVICE
        // Schluessel pruefen: Funktionsfehler
        if (key != val2) return 0x0600;

        int32_t result;
        // Parametergroesse setzen
        count = 0;

        // Wert an interne Struktur uebernehmen
        m_hardwareInfo.optionTPWrite = val;

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_TPWRITE, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

int32_t Datafox::Communication::ModuleHardware::writeHttpOption( uint8_t deviceAddress, uint8_t val, uint16_t val2  )   
{
    (void)val2;
    trace("%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceAddress, val);

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    uint16_t key;
    switch ( val )
    {
    case 0xFF:
        key = 0xCB78;
        break;

    case 0x00:
        key = 0x8B38;
        break;

    case 0x01:
        key = 0x4BF9;
        break;

    default:
        return -IV_ERROR_FUNCTION_PARAMETER;
    }

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );

        // Die Komponendenangabe uebernehmen.
        m_packetData[0] = val;
        count += sizeof(uint8_t);

        *(uint16_t*)&m_packetData[1] = key;
        count += sizeof(uint16_t);

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_HTTP, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceNumberReceiver(), IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_HTTP, result );
            return result;
        }

        // Keine Antwortdaten erwartet
        if ( count )
        {
            trace( "%s(%i):WARNING(3): %s() [ Antwort enthaelt %i Byte an unnuetze Daten. ]\n", __FILE__, __LINE__, __func__, count );
        }
    }
    else 
    {
#ifdef WITH_VIRTUAL_DEVICE
        // Schluessel pruefen: Funktionsfehler
        if (key != val2) return 0x0600;

        int32_t result;
        // Parametergroesse setzen
        count = 0;

        // Wert an interne Struktur uebernehmen
        m_hardwareInfo.optionHttp = val;

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_HTTP, &val, count)) != IV_ERROR_SUCCESS)
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

int32_t Datafox::Communication::ModuleHardware::writeLocalizationOption( uint8_t deviceAddress, uint8_t val, uint16_t val2 )
{ 
    (void)val; 
    (void)val2; 
    if ( !isOwnDeviceNumber( deviceAddress ) )
        return -IV_ERROR_FUNCTION_UNAVAILABLE;
    
    return 0x0200;
}

int32_t Datafox::Communication::ModuleHardware::writeFingerprintScannerUpdateOption( uint8_t deviceAddress, uint8_t val, uint16_t val2 )
{
    (void)val2;
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceAddress, val );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    uint16_t key;
    switch ( val )
    {
    case 0xFF:
        key = 0x0A39;
        break;

    case 0x00:
        key = 0x4A79;
        break;

    case 0x01:
        key = 0x8AB8;
        break;

    default:
        return -IV_ERROR_FUNCTION_PARAMETER;
    }

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );

        // Die Komponendenangabe uebernehmen.
        m_packetData[0] = val;
        count += sizeof(uint8_t);

        *(uint16_t*)&m_packetData[1] = key;
        count += sizeof(uint16_t);

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_BIOKEY_UPDATE, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceNumberReceiver(), IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_BIOKEY_UPDATE, result );
            return result;
        }

        // Keine Antwortdaten erwartet
        if ( count )
        {
            trace( "%s(%i):WARNING(3): %s() [ Antwort enthaelt %i Byte an unnuetze Daten. ]\n", __FILE__, __LINE__, __func__, count );
        }
    }
    else 
    {
#ifdef WITH_VIRTUAL_DEVICE
        // Schluessel pruefen: Funktionsfehler
        if ( key != val2 )
            return 0x0600;

        int32_t result;
        // Parametergroesse setzen
        count = 0;

        // Wert an interne Struktur uebernehmen
        m_hardwareInfo.optionBiokeyUpdate = val;

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_BIOKEY_UPDATE, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace("%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}
int32_t Datafox::Communication::ModuleHardware::writeBuyOptions( uint8_t deviceAddress, uint8_t mask, uint8_t options, uint16_t val2 )
{ 
    (void)mask; 
    (void)options; 
    (void)val2; 
    if ( !isOwnDeviceNumber( deviceAddress ) )
        return -IV_ERROR_FUNCTION_UNAVAILABLE;
    
    return 0x0200;
}

int32_t Datafox::Communication::ModuleHardware::writeHardwareInfoVersion( uint8_t deviceAddress, uint8_t val, uint16_t val2 )
{
    (void)val;
    (void)val2;
    if ( !isOwnDeviceNumber( deviceAddress ) )
        return -IV_ERROR_FUNCTION_UNAVAILABLE;
    
    return 0x0200;
}

#endif
