#include "file/dfcom_ini_data.h"
#include "file/dfcom_ini_helper.h"
#include "SimpleIni.h"

#include "fmt/format.h"

#include <boost/algorithm/string.hpp>

#if defined(_MSC_VER) || defined(__MINGW32__)

#ifndef WIN32_LEAN_AND_MEAN
#define WIN32_LEAN_AND_MEAN
#endif
#include <windows.h>
#include <direct.h>
#include <io.h>

#define fullpath             _fullpath
#define mkdir                _mkdir
#define stat                 _stat
#define finddata_t           _finddata_t
#define findfirst            _findfirst
#define findnext             _findnext
#define findclose            _findclose
#define snprintf	         _snprintf
#define access               _access

#endif

#if !defined(_MSC_VER)
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <dirent.h>
#endif

void DFComIniData::readData( const std::string &filePath )
{
    std::string section;
    std::string iniFilePath( filePath );
    if ( !iniFilePath.size() )
    {
        iniFilePath = DFComIniHelper::findIniFile();
    }

    if ( !iniFilePath.size() )
    {
        // Fehler bei Ermittlung
        m_enabled = false;
        m_traceMode = 0; // Dann auch Trace deaktivieren.
        m_eraseFiles = false;
        m_writeList = 0;
        return;
    }

    // Laden der Daten aus der Inidatei.
    CSimpleIniA ini;
    if ( ini.LoadFile( iniFilePath.c_str() ) != SI_OK )
    {
        return;
    }
    

    
    //section = fmt::format( "Docking_{}", m_connection_id );
    section = fmt::format( "Docking_{}", 1 ); // !!!! TODO: Erstmal f3st auf 1 gesetzt !!!!
    // Bevorzugt Dockingstation V1/2 ansprechen.
    m_dockingPreferType = ini.GetLongValue( section.c_str(), "Prefer", 1234 );
    if ( m_dockingPreferType == 1234 ) // Ggf. globale Einstellung laden.
    {
        m_dockingPreferType = ini.GetLongValue( "Docking", "Prefer", 2 );
    }

    if ( m_dockingPreferType < 1 )
    {
        m_dockingPreferType = 1;
    }
    else if ( m_dockingPreferType > 2 )
    {
        m_dockingPreferType = 2;
    }


    // Log aktiv?
    m_enabled = ini.GetBoolValue( "Log", "Enable", true );
    //m_enabled = (m_log_ini.GetUInt( "Log", "Enable", 1, ini_file_path.c_str() )) ? true : false;
    /*
    if ( !m_enabled )
    {
        // Abbruch durch Deaktivierung.
        m_traceMode = 0; // Dann auch Trace deaktivieren.
        m_eraseFiles = false;
        m_writeList = 0;
        return;
    }
    */
    // Trace aktiv?
    //m_trace_mode = m_log_ini.GetUInt( "Log", "Trace", 0, ini_file_path.c_str() );
    m_traceMode = ini.GetLongValue( "Log", "Trace", 0 );

    // Basispfad fuer Log
    m_logBasePath = ini.GetValue( "Log", "Path", "" );
    if ( !m_logBasePath.size() )
    {
        std::string logBasePath( iniFilePath );
        auto pos = logBasePath.find_last_of( '/' );
        if ( pos != std::string::npos )
        {
            logBasePath.resize( pos );
        }
        // Verzeichnis der Ini als Basis
        m_logBasePath = logBasePath;
    }
    else
    {
        std::string logBasePath( m_logBasePath );
#if defined(_MSC_VER) || defined(__MINGW32__)
        // Windows akzeptiert auch '/' als Pfadtrenner, somit Gemeinsamkeit mit GNU/Linux herstellen.
        boost::replace_all( logBasePath, "\\", "/" );
#endif
        if ( '/' == logBasePath[logBasePath.size() - 1] )
        {
            logBasePath[logBasePath.size() - 1] = 0;
        }

        // Aufbereitetes Verzeichnis als Basis
        m_logBasePath = logBasePath;
    }

    // Unterordner fuer Logdateien
    m_logSubFolder = ini.GetValue( "Log", "Folder", SUB_FOLDER );

    std::string logPath( m_logBasePath );
    logPath += '/';
    logPath += m_logSubFolder;
    if ( '/' == logPath[logPath.size() - 1] )
    {
        logPath[logPath.size() - 1] = 0;
    }

    // pruefen und evnt. anlegen des Ordners
    struct stat lstat;
    if ( stat( logPath.c_str(), &lstat ) != 0 || (lstat.st_mode & S_IFMT) != S_IFDIR )
    {
#if defined(_MSC_VER) || defined(__MINGW32__)
        if ( mkdir( logPath.c_str() ) != 0 )
        {
#else
        if ( mkdir( logPath.c_str(), S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH ) != 0 )
        {
#endif
            // Fehler bei Ermittlung
            m_enabled = false;
            m_traceMode = 0; // Dann auch Trace deaktivieren.
            m_eraseFiles = false;
            m_writeList = 0;
            return;
        }
        }

        // Schreibrecht prüfen.
    if ( access( logPath.c_str(), 2 ) == -1 )
    {
        // Fehler bei Ermittlung
        m_enabled = false;
        m_traceMode = 0; // Dann auch Trace deaktivieren.
        m_eraseFiles = false;
        m_writeList = 0;
        return;
    }

    // Alte Dateien loeschen
    m_eraseFiles = ini.GetBoolValue( "Log", "Erase", true );

    m_eraseFileDays = ini.GetLongValue( "Log", "Keep", 10 );
    if ( m_eraseFileDays < 1 )
    {
        m_eraseFileDays = 1;
    }
    if ( m_eraseFileDays > 100 )
    {
        m_eraseFileDays = 100;
    }

    // Ausfuehrungszeit angeben
    m_elapse = ini.GetBoolValue( "Log", "Elapse", true );

    // Listen wegschreiben
    m_writeList = ini.GetLongValue( "Log", "List", 1 );

    // Sektion [Channels]
    // ----------------------
    // Oeffnen und Schliessen muss durch den selben Thread erfolgen.
    m_checkOpenAndCloseBySameThread = ini.GetBoolValue( "Channels", "CheckOpenCloseBySameThread", true );

    // Dateinamen für Log und Trace setzen, alte Logdateien ggf. loeschen.
    //check_current_file_paths();

    // read configured documentation settings (path, activation state, ...)
    m_docPath = ini.GetValue( "docPath", "XML", "" );
    m_createXMLHelpFile = ini.GetBoolValue( "createHelpFile", "XML", false );
    
}
